TEST_FILE = "./tests/test_data/schema.k"


def test_exec_api():
    """Execute KCL file with arguments and return the JSON/YAML result."""
    import kcl_lib.api as api

    # Call the `exec_program` method with appropriate arguments
    args = api.ExecProgram_Args(k_filename_list=[TEST_FILE])
    # Usage
    api = api.API()
    result = api.exec_program(args)
    assert result.yaml_result == "app:\n  replicas: 2"


def test_exec_api_failed():
    """Execute KCL file with arguments and return the JSON/YAML result."""
    import kcl_lib.api as api

    try:
        # Call the `exec_program` method with appropriate arguments
        args = api.ExecProgram_Args(k_filename_list=["file_not_found"])
        # Usage
        api = api.API()
        result = api.exec_program(args)
        assert False
    except Exception as err:
        assert "Cannot find the kcl file" in str(err)


def test_parse_program_api():
    """Parse KCL program with entry files and return the AST JSON string."""
    import kcl_lib.api as api

    args = api.ParseProgram_Args(paths=[TEST_FILE])
    api = api.API()
    result = api.parse_program(args)
    assert len(result.paths) == 1
    assert len(result.errors) == 0


def test_parse_file_api():
    """Parse KCL single file to Module AST JSON string with import dependencies and parse errors."""
    import kcl_lib.api as api

    args = api.ParseFile_Args(path=TEST_FILE)
    api = api.API()
    result = api.parse_file(args)
    assert len(result.deps) == 0
    assert len(result.errors) == 0


def test_load_package_api():
    """load_package provides users with the ability to parse KCL program and semantic model
    information including symbols, types, definitions, etc.
    """
    import kcl_lib.api as api

    # Call the `load_package` method with appropriate arguments
    args = api.LoadPackage_Args(
        parse_args=api.ParseProgram_Args(paths=[TEST_FILE]), resolve_ast=True
    )
    # Usage
    api = api.API()
    result = api.load_package(args)
    symbols = list(result.symbols.values())
    assert len([s for s in symbols if s.ty.schema_name == "AppConfig"]) > 0


def test_list_variables_api():
    """list_variables provides users with the ability to parse KCL program and get all variables by specs."""
    import kcl_lib.api as api

    # Call the `list_variable` method with appropriate arguments
    args = api.ListVariables_Args(files=[TEST_FILE])
    # Usage
    api = api.API()
    result = api.list_variables(args)
    assert (
        result.variables["app"].variables[0].value == "AppConfig {\n    replicas: 2\n}"
    )


def test_list_options_api():
    """list_options provides users with the ability to parse KCL program and get all option information."""
    import kcl_lib.api as api

    args = api.ParseProgram_Args(paths=["./tests/test_data/option/main.k"])
    api = api.API()
    result = api.list_options(args)
    assert len(result.options) == 3
    assert result.options[0].name == "key1"
    assert result.options[1].name == "key2"
    assert result.options[2].name == "metadata-key"


def test_get_schema_type_api():
    """Get schema type mapping defined in the program."""
    import kcl_lib.api as api

    exec_args = api.ExecProgram_Args(k_filename_list=[TEST_FILE])
    # Call the `get_schema_type_mapping` method with appropriate arguments
    args = api.GetSchemaTypeMapping_Args(exec_args=exec_args)
    # Usage
    api = api.API()
    result = api.get_schema_type_mapping(args)
    assert result.schema_type_mapping["app"].properties["replicas"].type == "int"


def test_override_file_api():
    """Override KCL file with arguments. See https://www.kcl-lang.io/docs/user_docs/guides/automation
    for more override spec guide.
    """
    import kcl_lib.api as api
    import pathlib

    bak_file = "./tests/test_data/override_file/main.bak"
    test_file = "./tests/test_data/override_file/main.k"

    pathlib.Path(test_file).write_text(pathlib.Path(bak_file).read_text())

    args = api.OverrideFile_Args(
        file=test_file,
        specs=["b.a=2"],
    )
    api = api.API()
    result = api.override_file(args)
    assert len(result.parse_errors) == 0
    assert result.result == True
    assert (
        pathlib.Path(test_file).read_text()
        == """\
a = 1
b = {
    "a": 2
    "b": 2
}
"""
    )


def test_format_code_api():
    """Format the code source."""
    import kcl_lib.api as api

    source_code = """\
schema Person:
    name:   str
    age:    int

    check:
        0 <   age <   120
"""

    args = api.FormatCode_Args(source=source_code)

    api_instance = api.API()
    result = api_instance.format_code(args)

    assert (
        result.formatted.decode()
        == """\
schema Person:
    name: str
    age: int

    check:
        0 < age < 120

"""
    )


def test_format_path_api():
    """Format KCL file or directory path contains KCL files and returns the changed file paths."""
    import kcl_lib.api as api

    TEST_PATH = "./tests/test_data/format_path/test.k"

    args = api.FormatPath_Args(path=TEST_PATH)

    api_instance = api.API()
    result = api_instance.format_path(args)

    assert result.changed_paths == []


def test_lint_path_api():
    """Lint files and return error messages including errors and warnings."""
    import kcl_lib.api as api

    TEST_PATH = "./tests/test_data/lint_path/test-lint.k"

    args = api.LintPath_Args(paths=[TEST_PATH])

    api_instance = api.API()
    result = api_instance.lint_path(args)

    assert "Module 'math' imported but unused" in result.results


def test_validate_code_api():
    """Validate code using schema and JSON/YAML data strings."""
    import kcl_lib.api as api

    code = """\
schema Person:
    name: str
    age: int

    check:
        0 < age < 120
"""
    data = '{"name": "Alice", "age": 10}'

    args = api.ValidateCode_Args(code=code, data=data, format="json")

    api_instance = api.API()
    result = api_instance.validate_code(args)

    assert result.success == True
    assert result.err_message == ""


def test_rename_api():
    """Rename all the occurrences of the target symbol in the files. This API will rewrite files if they contain symbols to be renamed.
    Return the file paths that got changed.
    """
    import kcl_lib.api as api
    import pathlib

    pathlib.Path("./tests/test_data/rename/main.k").write_text(
        pathlib.Path("./tests/test_data/rename/main.bak").read_text()
    )

    args = api.Rename_Args(
        package_root="./tests/test_data/rename",
        symbol_path="a",
        file_paths=["./tests/test_data/rename/main.k"],
        new_name="a2",
    )

    api_instance = api.API()
    result = api_instance.rename(args)

    assert "tests/test_data/rename/main.k" in result.changed_files[0]


def test_rename_code_api():
    """Rename all the occurrences of the target symbol and return the modified code if any code has been changed.
    This API won't rewrite files but return the changed code.
    """
    import kcl_lib.api as api

    args = api.RenameCode_Args(
        package_root="/mock/path",
        symbol_path="a",
        source_codes={"/mock/path/main.k": "a = 1\nb = a"},
        new_name="a2",
    )

    api_instance = api.API()
    result = api_instance.rename_code(args)

    assert result.changed_codes["/mock/path/main.k"] == "a2 = 1\nb = a2"


def test_testing_api():
    """Test KCL packages with test arguments."""
    import kcl_lib.api as api

    args = api.Test_Args(
        pkg_list=["./tests/test_data/testing/..."],
    )

    api_instance = api.API()
    result = api_instance.test(args)

    assert len(result.info) == 2


def test_load_settings_files_api():
    """Load the setting file config defined in `kcl.yaml`"""
    import kcl_lib.api as api

    args = api.LoadSettingsFiles_Args(
        work_dir="./tests/test_data", files=["./tests/test_data/settings/kcl.yaml"]
    )

    api_instance = api.API()
    result = api_instance.load_settings_files(args)

    assert result.kcl_cli_configs.files == []
    assert result.kcl_cli_configs.strict_range_check == True
    assert (
        result.kcl_options[0].key == "key" and result.kcl_options[0].value == '"value"'
    )


def test_update_dependencies_api():
    """Download and update dependencies defined in the `kcl.mod` file and return the
    external package name and location list.
    """
    import kcl_lib.api as api

    args = api.UpdateDependencies_Args(
        manifest_path="./tests/test_data/update_dependencies"
    )

    api_instance = api.API()
    result = api_instance.update_dependencies(args)
    pkg_names = [pkg.pkg_name for pkg in result.external_pkgs]

    assert len(pkg_names) == 2
    assert "helloworld" in pkg_names
    assert "flask" in pkg_names


def test_exec_api_with_external_dependencies():
    import kcl_lib.api as api

    args = api.UpdateDependencies_Args(
        manifest_path="./tests/test_data/update_dependencies"
    )

    api_instance = api.API()
    result = api_instance.update_dependencies(args)
    exec_args = api.ExecProgram_Args(
        k_filename_list=["./tests/test_data/update_dependencies/main.k"],
        external_pkgs=result.external_pkgs,
    )
    result = api_instance.exec_program(exec_args)
    assert result.yaml_result == "a: Hello World!"


def test_get_version_api():
    """Return the KCL service version information."""
    import kcl_lib.api as api

    api_instance = api.API()
    result = api_instance.get_version()
    assert "Version" in str(result) and "GitCommit" in str(result)



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class ExternalPkg(_message.Message):
    __slots__ = ("pkg_name", "pkg_path")
    PKG_NAME_FIELD_NUMBER: _ClassVar[int]
    PKG_PATH_FIELD_NUMBER: _ClassVar[int]
    pkg_name: str
    pkg_path: str
    def __init__(
        self, pkg_name: _Optional[str] = ..., pkg_path: _Optional[str] = ...
    ) -> None: ...

class Argument(_message.Message):
    __slots__ = ("name", "value")
    NAME_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    name: str
    value: str
    def __init__(
        self, name: _Optional[str] = ..., value: _Optional[str] = ...
    ) -> None: ...

class Error(_message.Message):
    __slots__ = ("level", "code", "messages")
    LEVEL_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    level: str
    code: str
    messages: _containers.RepeatedCompositeFieldContainer[Message]
    def __init__(
        self,
        level: _Optional[str] = ...,
        code: _Optional[str] = ...,
        messages: _Optional[_Iterable[_Union[Message, _Mapping]]] = ...,
    ) -> None: ...

class Message(_message.Message):
    __slots__ = ("msg", "pos")
    MSG_FIELD_NUMBER: _ClassVar[int]
    POS_FIELD_NUMBER: _ClassVar[int]
    msg: str
    pos: Position
    def __init__(
        self,
        msg: _Optional[str] = ...,
        pos: _Optional[_Union[Position, _Mapping]] = ...,
    ) -> None: ...

class Ping_Args(_message.Message):
    __slots__ = ("value",)
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: str
    def __init__(self, value: _Optional[str] = ...) -> None: ...

class Ping_Result(_message.Message):
    __slots__ = ("value",)
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: str
    def __init__(self, value: _Optional[str] = ...) -> None: ...

class GetVersion_Args(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GetVersion_Result(_message.Message):
    __slots__ = ("version", "checksum", "git_sha", "version_info")
    VERSION_FIELD_NUMBER: _ClassVar[int]
    CHECKSUM_FIELD_NUMBER: _ClassVar[int]
    GIT_SHA_FIELD_NUMBER: _ClassVar[int]
    VERSION_INFO_FIELD_NUMBER: _ClassVar[int]
    version: str
    checksum: str
    git_sha: str
    version_info: str
    def __init__(
        self,
        version: _Optional[str] = ...,
        checksum: _Optional[str] = ...,
        git_sha: _Optional[str] = ...,
        version_info: _Optional[str] = ...,
    ) -> None: ...

class ListMethod_Args(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListMethod_Result(_message.Message):
    __slots__ = ("method_name_list",)
    METHOD_NAME_LIST_FIELD_NUMBER: _ClassVar[int]
    method_name_list: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, method_name_list: _Optional[_Iterable[str]] = ...) -> None: ...

class ParseFile_Args(_message.Message):
    __slots__ = ("path", "source", "external_pkgs")
    PATH_FIELD_NUMBER: _ClassVar[int]
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_PKGS_FIELD_NUMBER: _ClassVar[int]
    path: str
    source: str
    external_pkgs: _containers.RepeatedCompositeFieldContainer[ExternalPkg]
    def __init__(
        self,
        path: _Optional[str] = ...,
        source: _Optional[str] = ...,
        external_pkgs: _Optional[_Iterable[_Union[ExternalPkg, _Mapping]]] = ...,
    ) -> None: ...

class ParseFile_Result(_message.Message):
    __slots__ = ("ast_json", "deps", "errors")
    AST_JSON_FIELD_NUMBER: _ClassVar[int]
    DEPS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    ast_json: str
    deps: _containers.RepeatedScalarFieldContainer[str]
    errors: _containers.RepeatedCompositeFieldContainer[Error]
    def __init__(
        self,
        ast_json: _Optional[str] = ...,
        deps: _Optional[_Iterable[str]] = ...,
        errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
    ) -> None: ...

class ParseProgram_Args(_message.Message):
    __slots__ = ("paths", "sources", "external_pkgs")
    PATHS_FIELD_NUMBER: _ClassVar[int]
    SOURCES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_PKGS_FIELD_NUMBER: _ClassVar[int]
    paths: _containers.RepeatedScalarFieldContainer[str]
    sources: _containers.RepeatedScalarFieldContainer[str]
    external_pkgs: _containers.RepeatedCompositeFieldContainer[ExternalPkg]
    def __init__(
        self,
        paths: _Optional[_Iterable[str]] = ...,
        sources: _Optional[_Iterable[str]] = ...,
        external_pkgs: _Optional[_Iterable[_Union[ExternalPkg, _Mapping]]] = ...,
    ) -> None: ...

class ParseProgram_Result(_message.Message):
    __slots__ = ("ast_json", "paths", "errors")
    AST_JSON_FIELD_NUMBER: _ClassVar[int]
    PATHS_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    ast_json: str
    paths: _containers.RepeatedScalarFieldContainer[str]
    errors: _containers.RepeatedCompositeFieldContainer[Error]
    def __init__(
        self,
        ast_json: _Optional[str] = ...,
        paths: _Optional[_Iterable[str]] = ...,
        errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
    ) -> None: ...

class LoadPackage_Args(_message.Message):
    __slots__ = ("parse_args", "resolve_ast", "load_builtin", "with_ast_index")
    PARSE_ARGS_FIELD_NUMBER: _ClassVar[int]
    RESOLVE_AST_FIELD_NUMBER: _ClassVar[int]
    LOAD_BUILTIN_FIELD_NUMBER: _ClassVar[int]
    WITH_AST_INDEX_FIELD_NUMBER: _ClassVar[int]
    parse_args: ParseProgram_Args
    resolve_ast: bool
    load_builtin: bool
    with_ast_index: bool
    def __init__(
        self,
        parse_args: _Optional[_Union[ParseProgram_Args, _Mapping]] = ...,
        resolve_ast: bool = ...,
        load_builtin: bool = ...,
        with_ast_index: bool = ...,
    ) -> None: ...

class LoadPackage_Result(_message.Message):
    __slots__ = (
        "program",
        "paths",
        "parse_errors",
        "type_errors",
        "scopes",
        "symbols",
        "node_symbol_map",
        "symbol_node_map",
        "fully_qualified_name_map",
        "pkg_scope_map",
    )

    class ScopesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: Scope
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[Scope, _Mapping]] = ...,
        ) -> None: ...

    class SymbolsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: Symbol
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[Symbol, _Mapping]] = ...,
        ) -> None: ...

    class NodeSymbolMapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SymbolIndex
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[SymbolIndex, _Mapping]] = ...,
        ) -> None: ...

    class SymbolNodeMapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    class FullyQualifiedNameMapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SymbolIndex
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[SymbolIndex, _Mapping]] = ...,
        ) -> None: ...

    class PkgScopeMapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ScopeIndex
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[ScopeIndex, _Mapping]] = ...,
        ) -> None: ...

    PROGRAM_FIELD_NUMBER: _ClassVar[int]
    PATHS_FIELD_NUMBER: _ClassVar[int]
    PARSE_ERRORS_FIELD_NUMBER: _ClassVar[int]
    TYPE_ERRORS_FIELD_NUMBER: _ClassVar[int]
    SCOPES_FIELD_NUMBER: _ClassVar[int]
    SYMBOLS_FIELD_NUMBER: _ClassVar[int]
    NODE_SYMBOL_MAP_FIELD_NUMBER: _ClassVar[int]
    SYMBOL_NODE_MAP_FIELD_NUMBER: _ClassVar[int]
    FULLY_QUALIFIED_NAME_MAP_FIELD_NUMBER: _ClassVar[int]
    PKG_SCOPE_MAP_FIELD_NUMBER: _ClassVar[int]
    program: str
    paths: _containers.RepeatedScalarFieldContainer[str]
    parse_errors: _containers.RepeatedCompositeFieldContainer[Error]
    type_errors: _containers.RepeatedCompositeFieldContainer[Error]
    scopes: _containers.MessageMap[str, Scope]
    symbols: _containers.MessageMap[str, Symbol]
    node_symbol_map: _containers.MessageMap[str, SymbolIndex]
    symbol_node_map: _containers.ScalarMap[str, str]
    fully_qualified_name_map: _containers.MessageMap[str, SymbolIndex]
    pkg_scope_map: _containers.MessageMap[str, ScopeIndex]
    def __init__(
        self,
        program: _Optional[str] = ...,
        paths: _Optional[_Iterable[str]] = ...,
        parse_errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
        type_errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
        scopes: _Optional[_Mapping[str, Scope]] = ...,
        symbols: _Optional[_Mapping[str, Symbol]] = ...,
        node_symbol_map: _Optional[_Mapping[str, SymbolIndex]] = ...,
        symbol_node_map: _Optional[_Mapping[str, str]] = ...,
        fully_qualified_name_map: _Optional[_Mapping[str, SymbolIndex]] = ...,
        pkg_scope_map: _Optional[_Mapping[str, ScopeIndex]] = ...,
    ) -> None: ...

class ListOptions_Result(_message.Message):
    __slots__ = ("options",)
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    options: _containers.RepeatedCompositeFieldContainer[OptionHelp]
    def __init__(
        self, options: _Optional[_Iterable[_Union[OptionHelp, _Mapping]]] = ...
    ) -> None: ...

class OptionHelp(_message.Message):
    __slots__ = ("name", "type", "required", "default_value", "help")
    NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VALUE_FIELD_NUMBER: _ClassVar[int]
    HELP_FIELD_NUMBER: _ClassVar[int]
    name: str
    type: str
    required: bool
    default_value: str
    help: str
    def __init__(
        self,
        name: _Optional[str] = ...,
        type: _Optional[str] = ...,
        required: bool = ...,
        default_value: _Optional[str] = ...,
        help: _Optional[str] = ...,
    ) -> None: ...

class Symbol(_message.Message):
    __slots__ = ("ty", "name", "owner", "attrs", "is_global")
    TY_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    DEF_FIELD_NUMBER: _ClassVar[int]
    ATTRS_FIELD_NUMBER: _ClassVar[int]
    IS_GLOBAL_FIELD_NUMBER: _ClassVar[int]
    ty: KclType
    name: str
    owner: SymbolIndex
    attrs: _containers.RepeatedCompositeFieldContainer[SymbolIndex]
    is_global: bool
    def __init__(
        self,
        ty: _Optional[_Union[KclType, _Mapping]] = ...,
        name: _Optional[str] = ...,
        owner: _Optional[_Union[SymbolIndex, _Mapping]] = ...,
        attrs: _Optional[_Iterable[_Union[SymbolIndex, _Mapping]]] = ...,
        is_global: bool = ...,
        **kwargs
    ) -> None: ...

class Scope(_message.Message):
    __slots__ = ("kind", "parent", "owner", "children", "defs")
    KIND_FIELD_NUMBER: _ClassVar[int]
    PARENT_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    CHILDREN_FIELD_NUMBER: _ClassVar[int]
    DEFS_FIELD_NUMBER: _ClassVar[int]
    kind: str
    parent: ScopeIndex
    owner: SymbolIndex
    children: _containers.RepeatedCompositeFieldContainer[ScopeIndex]
    defs: _containers.RepeatedCompositeFieldContainer[SymbolIndex]
    def __init__(
        self,
        kind: _Optional[str] = ...,
        parent: _Optional[_Union[ScopeIndex, _Mapping]] = ...,
        owner: _Optional[_Union[SymbolIndex, _Mapping]] = ...,
        children: _Optional[_Iterable[_Union[ScopeIndex, _Mapping]]] = ...,
        defs: _Optional[_Iterable[_Union[SymbolIndex, _Mapping]]] = ...,
    ) -> None: ...

class SymbolIndex(_message.Message):
    __slots__ = ("i", "g", "kind")
    I_FIELD_NUMBER: _ClassVar[int]
    G_FIELD_NUMBER: _ClassVar[int]
    KIND_FIELD_NUMBER: _ClassVar[int]
    i: int
    g: int
    kind: str
    def __init__(
        self,
        i: _Optional[int] = ...,
        g: _Optional[int] = ...,
        kind: _Optional[str] = ...,
    ) -> None: ...

class ScopeIndex(_message.Message):
    __slots__ = ("i", "g", "kind")
    I_FIELD_NUMBER: _ClassVar[int]
    G_FIELD_NUMBER: _ClassVar[int]
    KIND_FIELD_NUMBER: _ClassVar[int]
    i: int
    g: int
    kind: str
    def __init__(
        self,
        i: _Optional[int] = ...,
        g: _Optional[int] = ...,
        kind: _Optional[str] = ...,
    ) -> None: ...

class ExecProgram_Args(_message.Message):
    __slots__ = (
        "work_dir",
        "k_filename_list",
        "k_code_list",
        "args",
        "overrides",
        "disable_yaml_result",
        "print_override_ast",
        "strict_range_check",
        "disable_none",
        "verbose",
        "debug",
        "sort_keys",
        "external_pkgs",
        "include_schema_type_path",
        "compile_only",
        "show_hidden",
        "path_selector",
        "fast_eval",
    )
    WORK_DIR_FIELD_NUMBER: _ClassVar[int]
    K_FILENAME_LIST_FIELD_NUMBER: _ClassVar[int]
    K_CODE_LIST_FIELD_NUMBER: _ClassVar[int]
    ARGS_FIELD_NUMBER: _ClassVar[int]
    OVERRIDES_FIELD_NUMBER: _ClassVar[int]
    DISABLE_YAML_RESULT_FIELD_NUMBER: _ClassVar[int]
    PRINT_OVERRIDE_AST_FIELD_NUMBER: _ClassVar[int]
    STRICT_RANGE_CHECK_FIELD_NUMBER: _ClassVar[int]
    DISABLE_NONE_FIELD_NUMBER: _ClassVar[int]
    VERBOSE_FIELD_NUMBER: _ClassVar[int]
    DEBUG_FIELD_NUMBER: _ClassVar[int]
    SORT_KEYS_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_PKGS_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_SCHEMA_TYPE_PATH_FIELD_NUMBER: _ClassVar[int]
    COMPILE_ONLY_FIELD_NUMBER: _ClassVar[int]
    SHOW_HIDDEN_FIELD_NUMBER: _ClassVar[int]
    PATH_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    FAST_EVAL_FIELD_NUMBER: _ClassVar[int]
    work_dir: str
    k_filename_list: _containers.RepeatedScalarFieldContainer[str]
    k_code_list: _containers.RepeatedScalarFieldContainer[str]
    args: _containers.RepeatedCompositeFieldContainer[Argument]
    overrides: _containers.RepeatedScalarFieldContainer[str]
    disable_yaml_result: bool
    print_override_ast: bool
    strict_range_check: bool
    disable_none: bool
    verbose: int
    debug: int
    sort_keys: bool
    external_pkgs: _containers.RepeatedCompositeFieldContainer[ExternalPkg]
    include_schema_type_path: bool
    compile_only: bool
    show_hidden: bool
    path_selector: _containers.RepeatedScalarFieldContainer[str]
    fast_eval: bool
    def __init__(
        self,
        work_dir: _Optional[str] = ...,
        k_filename_list: _Optional[_Iterable[str]] = ...,
        k_code_list: _Optional[_Iterable[str]] = ...,
        args: _Optional[_Iterable[_Union[Argument, _Mapping]]] = ...,
        overrides: _Optional[_Iterable[str]] = ...,
        disable_yaml_result: bool = ...,
        print_override_ast: bool = ...,
        strict_range_check: bool = ...,
        disable_none: bool = ...,
        verbose: _Optional[int] = ...,
        debug: _Optional[int] = ...,
        sort_keys: bool = ...,
        external_pkgs: _Optional[_Iterable[_Union[ExternalPkg, _Mapping]]] = ...,
        include_schema_type_path: bool = ...,
        compile_only: bool = ...,
        show_hidden: bool = ...,
        path_selector: _Optional[_Iterable[str]] = ...,
        fast_eval: bool = ...,
    ) -> None: ...

class ExecProgram_Result(_message.Message):
    __slots__ = ("json_result", "yaml_result", "log_message", "err_message")
    JSON_RESULT_FIELD_NUMBER: _ClassVar[int]
    YAML_RESULT_FIELD_NUMBER: _ClassVar[int]
    LOG_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    ERR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    json_result: str
    yaml_result: str
    log_message: str
    err_message: str
    def __init__(
        self,
        json_result: _Optional[str] = ...,
        yaml_result: _Optional[str] = ...,
        log_message: _Optional[str] = ...,
        err_message: _Optional[str] = ...,
    ) -> None: ...

class BuildProgram_Args(_message.Message):
    __slots__ = ("exec_args", "output")
    EXEC_ARGS_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    exec_args: ExecProgram_Args
    output: str
    def __init__(
        self,
        exec_args: _Optional[_Union[ExecProgram_Args, _Mapping]] = ...,
        output: _Optional[str] = ...,
    ) -> None: ...

class BuildProgram_Result(_message.Message):
    __slots__ = ("path",)
    PATH_FIELD_NUMBER: _ClassVar[int]
    path: str
    def __init__(self, path: _Optional[str] = ...) -> None: ...

class ExecArtifact_Args(_message.Message):
    __slots__ = ("path", "exec_args")
    PATH_FIELD_NUMBER: _ClassVar[int]
    EXEC_ARGS_FIELD_NUMBER: _ClassVar[int]
    path: str
    exec_args: ExecProgram_Args
    def __init__(
        self,
        path: _Optional[str] = ...,
        exec_args: _Optional[_Union[ExecProgram_Args, _Mapping]] = ...,
    ) -> None: ...

class FormatCode_Args(_message.Message):
    __slots__ = ("source",)
    SOURCE_FIELD_NUMBER: _ClassVar[int]
    source: str
    def __init__(self, source: _Optional[str] = ...) -> None: ...

class FormatCode_Result(_message.Message):
    __slots__ = ("formatted",)
    FORMATTED_FIELD_NUMBER: _ClassVar[int]
    formatted: bytes
    def __init__(self, formatted: _Optional[bytes] = ...) -> None: ...

class FormatPath_Args(_message.Message):
    __slots__ = ("path",)
    PATH_FIELD_NUMBER: _ClassVar[int]
    path: str
    def __init__(self, path: _Optional[str] = ...) -> None: ...

class FormatPath_Result(_message.Message):
    __slots__ = ("changed_paths",)
    CHANGED_PATHS_FIELD_NUMBER: _ClassVar[int]
    changed_paths: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, changed_paths: _Optional[_Iterable[str]] = ...) -> None: ...

class LintPath_Args(_message.Message):
    __slots__ = ("paths",)
    PATHS_FIELD_NUMBER: _ClassVar[int]
    paths: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, paths: _Optional[_Iterable[str]] = ...) -> None: ...

class LintPath_Result(_message.Message):
    __slots__ = ("results",)
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, results: _Optional[_Iterable[str]] = ...) -> None: ...

class OverrideFile_Args(_message.Message):
    __slots__ = ("file", "specs", "import_paths")
    FILE_FIELD_NUMBER: _ClassVar[int]
    SPECS_FIELD_NUMBER: _ClassVar[int]
    IMPORT_PATHS_FIELD_NUMBER: _ClassVar[int]
    file: str
    specs: _containers.RepeatedScalarFieldContainer[str]
    import_paths: _containers.RepeatedScalarFieldContainer[str]
    def __init__(
        self,
        file: _Optional[str] = ...,
        specs: _Optional[_Iterable[str]] = ...,
        import_paths: _Optional[_Iterable[str]] = ...,
    ) -> None: ...

class OverrideFile_Result(_message.Message):
    __slots__ = ("result", "parse_errors")
    RESULT_FIELD_NUMBER: _ClassVar[int]
    PARSE_ERRORS_FIELD_NUMBER: _ClassVar[int]
    result: bool
    parse_errors: _containers.RepeatedCompositeFieldContainer[Error]
    def __init__(
        self,
        result: bool = ...,
        parse_errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
    ) -> None: ...

class ListVariables_Options(_message.Message):
    __slots__ = ("merge_program",)
    MERGE_PROGRAM_FIELD_NUMBER: _ClassVar[int]
    merge_program: bool
    def __init__(self, merge_program: bool = ...) -> None: ...

class VariableList(_message.Message):
    __slots__ = ("variables",)
    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.RepeatedCompositeFieldContainer[Variable]
    def __init__(
        self, variables: _Optional[_Iterable[_Union[Variable, _Mapping]]] = ...
    ) -> None: ...

class ListVariables_Args(_message.Message):
    __slots__ = ("files", "specs", "options")
    FILES_FIELD_NUMBER: _ClassVar[int]
    SPECS_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    files: _containers.RepeatedScalarFieldContainer[str]
    specs: _containers.RepeatedScalarFieldContainer[str]
    options: ListVariables_Options
    def __init__(
        self,
        files: _Optional[_Iterable[str]] = ...,
        specs: _Optional[_Iterable[str]] = ...,
        options: _Optional[_Union[ListVariables_Options, _Mapping]] = ...,
    ) -> None: ...

class ListVariables_Result(_message.Message):
    __slots__ = ("variables", "unsupported_codes", "parse_errors")

    class VariablesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: VariableList
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[VariableList, _Mapping]] = ...,
        ) -> None: ...

    VARIABLES_FIELD_NUMBER: _ClassVar[int]
    UNSUPPORTED_CODES_FIELD_NUMBER: _ClassVar[int]
    PARSE_ERRORS_FIELD_NUMBER: _ClassVar[int]
    variables: _containers.MessageMap[str, VariableList]
    unsupported_codes: _containers.RepeatedScalarFieldContainer[str]
    parse_errors: _containers.RepeatedCompositeFieldContainer[Error]
    def __init__(
        self,
        variables: _Optional[_Mapping[str, VariableList]] = ...,
        unsupported_codes: _Optional[_Iterable[str]] = ...,
        parse_errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...,
    ) -> None: ...

class Variable(_message.Message):
    __slots__ = ("value", "type_name", "op_sym", "list_items", "dict_entries")
    VALUE_FIELD_NUMBER: _ClassVar[int]
    TYPE_NAME_FIELD_NUMBER: _ClassVar[int]
    OP_SYM_FIELD_NUMBER: _ClassVar[int]
    LIST_ITEMS_FIELD_NUMBER: _ClassVar[int]
    DICT_ENTRIES_FIELD_NUMBER: _ClassVar[int]
    value: str
    type_name: str
    op_sym: str
    list_items: _containers.RepeatedCompositeFieldContainer[Variable]
    dict_entries: _containers.RepeatedCompositeFieldContainer[MapEntry]
    def __init__(
        self,
        value: _Optional[str] = ...,
        type_name: _Optional[str] = ...,
        op_sym: _Optional[str] = ...,
        list_items: _Optional[_Iterable[_Union[Variable, _Mapping]]] = ...,
        dict_entries: _Optional[_Iterable[_Union[MapEntry, _Mapping]]] = ...,
    ) -> None: ...

class MapEntry(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: Variable
    def __init__(
        self,
        key: _Optional[str] = ...,
        value: _Optional[_Union[Variable, _Mapping]] = ...,
    ) -> None: ...

class GetSchemaTypeMapping_Args(_message.Message):
    __slots__ = ("exec_args", "schema_name")
    EXEC_ARGS_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    exec_args: ExecProgram_Args
    schema_name: str
    def __init__(
        self,
        exec_args: _Optional[_Union[ExecProgram_Args, _Mapping]] = ...,
        schema_name: _Optional[str] = ...,
    ) -> None: ...

class GetSchemaTypeMapping_Result(_message.Message):
    __slots__ = ("schema_type_mapping",)

    class SchemaTypeMappingEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: KclType
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[KclType, _Mapping]] = ...,
        ) -> None: ...

    SCHEMA_TYPE_MAPPING_FIELD_NUMBER: _ClassVar[int]
    schema_type_mapping: _containers.MessageMap[str, KclType]
    def __init__(
        self, schema_type_mapping: _Optional[_Mapping[str, KclType]] = ...
    ) -> None: ...

class ValidateCode_Args(_message.Message):
    __slots__ = (
        "datafile",
        "data",
        "file",
        "code",
        "schema",
        "attribute_name",
        "format",
    )
    DATAFILE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    FILE_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    ATTRIBUTE_NAME_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    datafile: str
    data: str
    file: str
    code: str
    schema: str
    attribute_name: str
    format: str
    def __init__(
        self,
        datafile: _Optional[str] = ...,
        data: _Optional[str] = ...,
        file: _Optional[str] = ...,
        code: _Optional[str] = ...,
        schema: _Optional[str] = ...,
        attribute_name: _Optional[str] = ...,
        format: _Optional[str] = ...,
    ) -> None: ...

class ValidateCode_Result(_message.Message):
    __slots__ = ("success", "err_message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    ERR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    err_message: str
    def __init__(
        self, success: bool = ..., err_message: _Optional[str] = ...
    ) -> None: ...

class Position(_message.Message):
    __slots__ = ("line", "column", "filename")
    LINE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    FILENAME_FIELD_NUMBER: _ClassVar[int]
    line: int
    column: int
    filename: str
    def __init__(
        self,
        line: _Optional[int] = ...,
        column: _Optional[int] = ...,
        filename: _Optional[str] = ...,
    ) -> None: ...

class ListDepFiles_Args(_message.Message):
    __slots__ = ("work_dir", "use_abs_path", "include_all", "use_fast_parser")
    WORK_DIR_FIELD_NUMBER: _ClassVar[int]
    USE_ABS_PATH_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_ALL_FIELD_NUMBER: _ClassVar[int]
    USE_FAST_PARSER_FIELD_NUMBER: _ClassVar[int]
    work_dir: str
    use_abs_path: bool
    include_all: bool
    use_fast_parser: bool
    def __init__(
        self,
        work_dir: _Optional[str] = ...,
        use_abs_path: bool = ...,
        include_all: bool = ...,
        use_fast_parser: bool = ...,
    ) -> None: ...

class ListDepFiles_Result(_message.Message):
    __slots__ = ("pkgroot", "pkgpath", "files")
    PKGROOT_FIELD_NUMBER: _ClassVar[int]
    PKGPATH_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    pkgroot: str
    pkgpath: str
    files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(
        self,
        pkgroot: _Optional[str] = ...,
        pkgpath: _Optional[str] = ...,
        files: _Optional[_Iterable[str]] = ...,
    ) -> None: ...

class LoadSettingsFiles_Args(_message.Message):
    __slots__ = ("work_dir", "files")
    WORK_DIR_FIELD_NUMBER: _ClassVar[int]
    FILES_FIELD_NUMBER: _ClassVar[int]
    work_dir: str
    files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(
        self, work_dir: _Optional[str] = ..., files: _Optional[_Iterable[str]] = ...
    ) -> None: ...

class LoadSettingsFiles_Result(_message.Message):
    __slots__ = ("kcl_cli_configs", "kcl_options")
    KCL_CLI_CONFIGS_FIELD_NUMBER: _ClassVar[int]
    KCL_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    kcl_cli_configs: CliConfig
    kcl_options: _containers.RepeatedCompositeFieldContainer[KeyValuePair]
    def __init__(
        self,
        kcl_cli_configs: _Optional[_Union[CliConfig, _Mapping]] = ...,
        kcl_options: _Optional[_Iterable[_Union[KeyValuePair, _Mapping]]] = ...,
    ) -> None: ...

class CliConfig(_message.Message):
    __slots__ = (
        "files",
        "output",
        "overrides",
        "path_selector",
        "strict_range_check",
        "disable_none",
        "verbose",
        "debug",
        "sort_keys",
        "show_hidden",
        "include_schema_type_path",
        "fast_eval",
    )
    FILES_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    OVERRIDES_FIELD_NUMBER: _ClassVar[int]
    PATH_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    STRICT_RANGE_CHECK_FIELD_NUMBER: _ClassVar[int]
    DISABLE_NONE_FIELD_NUMBER: _ClassVar[int]
    VERBOSE_FIELD_NUMBER: _ClassVar[int]
    DEBUG_FIELD_NUMBER: _ClassVar[int]
    SORT_KEYS_FIELD_NUMBER: _ClassVar[int]
    SHOW_HIDDEN_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_SCHEMA_TYPE_PATH_FIELD_NUMBER: _ClassVar[int]
    FAST_EVAL_FIELD_NUMBER: _ClassVar[int]
    files: _containers.RepeatedScalarFieldContainer[str]
    output: str
    overrides: _containers.RepeatedScalarFieldContainer[str]
    path_selector: _containers.RepeatedScalarFieldContainer[str]
    strict_range_check: bool
    disable_none: bool
    verbose: int
    debug: bool
    sort_keys: bool
    show_hidden: bool
    include_schema_type_path: bool
    fast_eval: bool
    def __init__(
        self,
        files: _Optional[_Iterable[str]] = ...,
        output: _Optional[str] = ...,
        overrides: _Optional[_Iterable[str]] = ...,
        path_selector: _Optional[_Iterable[str]] = ...,
        strict_range_check: bool = ...,
        disable_none: bool = ...,
        verbose: _Optional[int] = ...,
        debug: bool = ...,
        sort_keys: bool = ...,
        show_hidden: bool = ...,
        include_schema_type_path: bool = ...,
        fast_eval: bool = ...,
    ) -> None: ...

class KeyValuePair(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(
        self, key: _Optional[str] = ..., value: _Optional[str] = ...
    ) -> None: ...

class Rename_Args(_message.Message):
    __slots__ = ("package_root", "symbol_path", "file_paths", "new_name")
    PACKAGE_ROOT_FIELD_NUMBER: _ClassVar[int]
    SYMBOL_PATH_FIELD_NUMBER: _ClassVar[int]
    FILE_PATHS_FIELD_NUMBER: _ClassVar[int]
    NEW_NAME_FIELD_NUMBER: _ClassVar[int]
    package_root: str
    symbol_path: str
    file_paths: _containers.RepeatedScalarFieldContainer[str]
    new_name: str
    def __init__(
        self,
        package_root: _Optional[str] = ...,
        symbol_path: _Optional[str] = ...,
        file_paths: _Optional[_Iterable[str]] = ...,
        new_name: _Optional[str] = ...,
    ) -> None: ...

class Rename_Result(_message.Message):
    __slots__ = ("changed_files",)
    CHANGED_FILES_FIELD_NUMBER: _ClassVar[int]
    changed_files: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, changed_files: _Optional[_Iterable[str]] = ...) -> None: ...

class RenameCode_Args(_message.Message):
    __slots__ = ("package_root", "symbol_path", "source_codes", "new_name")

    class SourceCodesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    PACKAGE_ROOT_FIELD_NUMBER: _ClassVar[int]
    SYMBOL_PATH_FIELD_NUMBER: _ClassVar[int]
    SOURCE_CODES_FIELD_NUMBER: _ClassVar[int]
    NEW_NAME_FIELD_NUMBER: _ClassVar[int]
    package_root: str
    symbol_path: str
    source_codes: _containers.ScalarMap[str, str]
    new_name: str
    def __init__(
        self,
        package_root: _Optional[str] = ...,
        symbol_path: _Optional[str] = ...,
        source_codes: _Optional[_Mapping[str, str]] = ...,
        new_name: _Optional[str] = ...,
    ) -> None: ...

class RenameCode_Result(_message.Message):
    __slots__ = ("changed_codes",)

    class ChangedCodesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    CHANGED_CODES_FIELD_NUMBER: _ClassVar[int]
    changed_codes: _containers.ScalarMap[str, str]
    def __init__(self, changed_codes: _Optional[_Mapping[str, str]] = ...) -> None: ...

class Test_Args(_message.Message):
    __slots__ = ("exec_args", "pkg_list", "run_regexp", "fail_fast")
    EXEC_ARGS_FIELD_NUMBER: _ClassVar[int]
    PKG_LIST_FIELD_NUMBER: _ClassVar[int]
    RUN_REGEXP_FIELD_NUMBER: _ClassVar[int]
    FAIL_FAST_FIELD_NUMBER: _ClassVar[int]
    exec_args: ExecProgram_Args
    pkg_list: _containers.RepeatedScalarFieldContainer[str]
    run_regexp: str
    fail_fast: bool
    def __init__(
        self,
        exec_args: _Optional[_Union[ExecProgram_Args, _Mapping]] = ...,
        pkg_list: _Optional[_Iterable[str]] = ...,
        run_regexp: _Optional[str] = ...,
        fail_fast: bool = ...,
    ) -> None: ...

class Test_Result(_message.Message):
    __slots__ = ("info",)
    INFO_FIELD_NUMBER: _ClassVar[int]
    info: _containers.RepeatedCompositeFieldContainer[TestCaseInfo]
    def __init__(
        self, info: _Optional[_Iterable[_Union[TestCaseInfo, _Mapping]]] = ...
    ) -> None: ...

class TestCaseInfo(_message.Message):
    __slots__ = ("name", "error", "duration", "log_message")
    NAME_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    LOG_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    name: str
    error: str
    duration: int
    log_message: str
    def __init__(
        self,
        name: _Optional[str] = ...,
        error: _Optional[str] = ...,
        duration: _Optional[int] = ...,
        log_message: _Optional[str] = ...,
    ) -> None: ...

class UpdateDependencies_Args(_message.Message):
    __slots__ = ("manifest_path", "vendor")
    MANIFEST_PATH_FIELD_NUMBER: _ClassVar[int]
    VENDOR_FIELD_NUMBER: _ClassVar[int]
    manifest_path: str
    vendor: bool
    def __init__(
        self, manifest_path: _Optional[str] = ..., vendor: bool = ...
    ) -> None: ...

class UpdateDependencies_Result(_message.Message):
    __slots__ = ("external_pkgs",)
    EXTERNAL_PKGS_FIELD_NUMBER: _ClassVar[int]
    external_pkgs: _containers.RepeatedCompositeFieldContainer[ExternalPkg]
    def __init__(
        self, external_pkgs: _Optional[_Iterable[_Union[ExternalPkg, _Mapping]]] = ...
    ) -> None: ...

class KclType(_message.Message):
    __slots__ = (
        "type",
        "union_types",
        "default",
        "schema_name",
        "schema_doc",
        "properties",
        "required",
        "key",
        "item",
        "line",
        "decorators",
        "filename",
        "pkg_path",
        "description",
        "examples",
        "base_schema",
    )

    class PropertiesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: KclType
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[KclType, _Mapping]] = ...,
        ) -> None: ...

    class ExamplesEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: Example
        def __init__(
            self,
            key: _Optional[str] = ...,
            value: _Optional[_Union[Example, _Mapping]] = ...,
        ) -> None: ...

    TYPE_FIELD_NUMBER: _ClassVar[int]
    UNION_TYPES_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_DOC_FIELD_NUMBER: _ClassVar[int]
    PROPERTIES_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_FIELD_NUMBER: _ClassVar[int]
    KEY_FIELD_NUMBER: _ClassVar[int]
    ITEM_FIELD_NUMBER: _ClassVar[int]
    LINE_FIELD_NUMBER: _ClassVar[int]
    DECORATORS_FIELD_NUMBER: _ClassVar[int]
    FILENAME_FIELD_NUMBER: _ClassVar[int]
    PKG_PATH_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXAMPLES_FIELD_NUMBER: _ClassVar[int]
    BASE_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    type: str
    union_types: _containers.RepeatedCompositeFieldContainer[KclType]
    default: str
    schema_name: str
    schema_doc: str
    properties: _containers.MessageMap[str, KclType]
    required: _containers.RepeatedScalarFieldContainer[str]
    key: KclType
    item: KclType
    line: int
    decorators: _containers.RepeatedCompositeFieldContainer[Decorator]
    filename: str
    pkg_path: str
    description: str
    examples: _containers.MessageMap[str, Example]
    base_schema: KclType
    def __init__(
        self,
        type: _Optional[str] = ...,
        union_types: _Optional[_Iterable[_Union[KclType, _Mapping]]] = ...,
        default: _Optional[str] = ...,
        schema_name: _Optional[str] = ...,
        schema_doc: _Optional[str] = ...,
        properties: _Optional[_Mapping[str, KclType]] = ...,
        required: _Optional[_Iterable[str]] = ...,
        key: _Optional[_Union[KclType, _Mapping]] = ...,
        item: _Optional[_Union[KclType, _Mapping]] = ...,
        line: _Optional[int] = ...,
        decorators: _Optional[_Iterable[_Union[Decorator, _Mapping]]] = ...,
        filename: _Optional[str] = ...,
        pkg_path: _Optional[str] = ...,
        description: _Optional[str] = ...,
        examples: _Optional[_Mapping[str, Example]] = ...,
        base_schema: _Optional[_Union[KclType, _Mapping]] = ...,
    ) -> None: ...

class Decorator(_message.Message):
    __slots__ = ("name", "arguments", "keywords")

    class KeywordsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[str] = ...
        ) -> None: ...

    NAME_FIELD_NUMBER: _ClassVar[int]
    ARGUMENTS_FIELD_NUMBER: _ClassVar[int]
    KEYWORDS_FIELD_NUMBER: _ClassVar[int]
    name: str
    arguments: _containers.RepeatedScalarFieldContainer[str]
    keywords: _containers.ScalarMap[str, str]
    def __init__(
        self,
        name: _Optional[str] = ...,
        arguments: _Optional[_Iterable[str]] = ...,
        keywords: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class Example(_message.Message):
    __slots__ = ("summary", "description", "value")
    SUMMARY_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    summary: str
    description: str
    value: str
    def __init__(
        self,
        summary: _Optional[str] = ...,
        description: _Optional[str] = ...,
        value: _Optional[str] = ...,
    ) -> None: ...



import kcl_lib.plugin as plugin
import kcl_lib.api as api

TEST_FILE = "./tests/test_data/plugin.k"


def test_exec_api():
    plugin.register_plugin("my_plugin", {"add": lambda x, y: x + y})
    result = api.API().exec_program(api.ExecProgram_Args(k_filename_list=[TEST_FILE]))
    assert result.yaml_result == "result: 2"


def test_exec_api_failed():
    error_msg = "plugin error"

    def func(x, y):
        raise ValueError(error_msg)

    plugin.register_plugin("my_plugin", {"add": func})
    result = api.API().exec_program(api.ExecProgram_Args(k_filename_list=[TEST_FILE]))
    assert error_msg in result.err_message



import json
import typing
from ctypes import *
from dataclasses import dataclass


@dataclass
class Plugin:
    name: str
    method_map: typing.Dict[str, typing.Callable]


class PluginContext:
    def __init__(self):
        self._plugin_dict: typing.Dict[str, Plugin] = {}

    def call_method(self, name: str, args_json: str, kwargs_json: str) -> str:
        return self._call_py_method(name, args_json, kwargs_json)

    def _call_py_method(self, name: str, args_json: str, kwargs_json: str) -> str:
        try:
            return self._call_py_method_unsafe(name, args_json, kwargs_json)
        except Exception as e:
            return json.dumps({"__kcl_PanicInfo__": f"{e}"})

    def _call_py_method_unsafe(
        self, name: str, args_json: str, kwargs_json: str
    ) -> str:
        dotIdx = name.rfind(".")
        if dotIdx < 0:
            return ""

        # kcl_plugin.<module_path>.<method_name>
        module_path = name[:dotIdx]
        method_name = name[dotIdx + 1 :]
        plugin_name = module_path[module_path.rfind(".") + 1 :]

        method_func = self._plugin_dict.get(
            plugin_name, Plugin(name="", method_map={})
        ).method_map.get(method_name, None)

        args = []
        kwargs = {}

        if args_json:
            args = json.loads(args_json)
            if not isinstance(args, list):
                return ""

        if kwargs_json:
            kwargs = json.loads(kwargs_json)
            if not isinstance(kwargs, dict):
                return ""

        result = method_func(*args, **kwargs) if method_func else None
        return json.dumps(result)


__plugin_context__ = PluginContext()
__plugin_method_agent_buffer__ = create_string_buffer(1024)


def register_plugin(
    name: str, method_map: typing.Dict[str, typing.Callable]
) -> c_char_p:
    __plugin_context__._plugin_dict[name] = Plugin(name, method_map)


@CFUNCTYPE(c_char_p, c_char_p, c_char_p, c_char_p)
def plugin_method_agent(method: str, args_json: str, kwargs_json: str) -> c_char_p:
    method = str(method, encoding="utf-8")
    args_json = str(args_json, encoding="utf-8")
    kwargs_json = str(kwargs_json, encoding="utf-8")

    json_result = __plugin_context__.call_method(method, args_json, kwargs_json)

    global __plugin_method_agent_buffer__
    __plugin_method_agent_buffer__ = create_string_buffer(json_result.encode("utf-8"))
    return addressof(__plugin_method_agent_buffer__)


plugin_agent_addr = cast(plugin_method_agent, c_void_p).value



"""KCL Python API"""

from .spec_pb2 import (
    ExternalPkg,
    Argument,
    Error,
    Message,
    Ping_Args,
    Ping_Result,
    GetVersion_Args,
    GetVersion_Result,
    ListMethod_Args,
    ListMethod_Result,
    ParseFile_Args,
    ParseFile_Result,
    ParseProgram_Args,
    ParseProgram_Result,
    LoadPackage_Args,
    LoadPackage_Result,
    ListOptions_Result,
    OptionHelp,
    Symbol,
    Scope,
    SymbolIndex,
    ScopeIndex,
    ExecProgram_Args,
    ExecProgram_Result,
    BuildProgram_Args,
    BuildProgram_Result,
    ExecArtifact_Args,
    FormatCode_Args,
    FormatCode_Result,
    FormatPath_Args,
    FormatPath_Result,
    LintPath_Args,
    LintPath_Result,
    OverrideFile_Args,
    OverrideFile_Result,
    ListVariables_Options,
    VariableList,
    ListVariables_Args,
    ListVariables_Result,
    Variable,
    MapEntry,
    GetSchemaTypeMapping_Args,
    GetSchemaTypeMapping_Result,
    ValidateCode_Args,
    ValidateCode_Result,
    Position,
    ListDepFiles_Args,
    ListDepFiles_Result,
    LoadSettingsFiles_Args,
    LoadSettingsFiles_Result,
    CliConfig,
    KeyValuePair,
    Rename_Args,
    Rename_Result,
    RenameCode_Args,
    RenameCode_Result,
    Test_Args,
    Test_Result,
    TestCaseInfo,
    UpdateDependencies_Args,
    UpdateDependencies_Result,
    KclType,
    Decorator,
    Example,
)
from .service import API

__all__ = [
    "API",
    "ExternalPkg",
    "Argument",
    "Error",
    "Message",
    "Ping_Args",
    "Ping_Result",
    "GetVersion_Args",
    "GetVersion_Result",
    "ListMethod_Args",
    "ListMethod_Result",
    "ParseFile_Args",
    "ParseFile_Result",
    "ParseProgram_Args",
    "ParseProgram_Result",
    "LoadPackage_Args",
    "LoadPackage_Result",
    "ListOptions_Result",
    "OptionHelp",
    "Symbol",
    "Scope",
    "SymbolIndex",
    "ScopeIndex",
    "ExecProgram_Args",
    "ExecProgram_Result",
    "BuildProgram_Args",
    "BuildProgram_Result",
    "ExecArtifact_Args",
    "FormatCode_Args",
    "FormatCode_Result",
    "FormatPath_Args",
    "FormatPath_Result",
    "LintPath_Args",
    "LintPath_Result",
    "OverrideFile_Args",
    "OverrideFile_Result",
    "ListVariables_Options",
    "VariableList",
    "ListVariables_Args",
    "ListVariables_Result",
    "Variable",
    "MapEntry",
    "GetSchemaTypeMapping_Args",
    "GetSchemaTypeMapping_Result",
    "ValidateCode_Args",
    "ValidateCode_Result",
    "Position",
    "ListDepFiles_Args",
    "ListDepFiles_Result",
    "LoadSettingsFiles_Args",
    "LoadSettingsFiles_Result",
    "CliConfig",
    "KeyValuePair",
    "Rename_Args",
    "Rename_Result",
    "RenameCode_Args",
    "RenameCode_Result",
    "Test_Args",
    "Test_Result",
    "TestCaseInfo",
    "UpdateDependencies_Args",
    "UpdateDependencies_Result",
    "KclType",
    "Decorator",
    "Example",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: spec.proto
# Protobuf Python Version: 5.27.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 5, 27, 0, "", "spec.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\nspec.proto\x12\x0b\x63om.kcl.api"1\n\x0b\x45xternalPkg\x12\x10\n\x08pkg_name\x18\x01 \x01(\t\x12\x10\n\x08pkg_path\x18\x02 \x01(\t"\'\n\x08\x41rgument\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"L\n\x05\x45rror\x12\r\n\x05level\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12&\n\x08messages\x18\x03 \x03(\x0b\x32\x14.com.kcl.api.Message":\n\x07Message\x12\x0b\n\x03msg\x18\x01 \x01(\t\x12"\n\x03pos\x18\x02 \x01(\x0b\x32\x15.com.kcl.api.Position"\x1a\n\tPing_Args\x12\r\n\x05value\x18\x01 \x01(\t"\x1c\n\x0bPing_Result\x12\r\n\x05value\x18\x01 \x01(\t"\x11\n\x0fGetVersion_Args"]\n\x11GetVersion_Result\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x10\n\x08\x63hecksum\x18\x02 \x01(\t\x12\x0f\n\x07git_sha\x18\x03 \x01(\t\x12\x14\n\x0cversion_info\x18\x04 \x01(\t"\x11\n\x0fListMethod_Args"-\n\x11ListMethod_Result\x12\x18\n\x10method_name_list\x18\x01 \x03(\t"_\n\x0eParseFile_Args\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06source\x18\x02 \x01(\t\x12/\n\rexternal_pkgs\x18\x03 \x03(\x0b\x32\x18.com.kcl.api.ExternalPkg"V\n\x10ParseFile_Result\x12\x10\n\x08\x61st_json\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x65ps\x18\x02 \x03(\t\x12"\n\x06\x65rrors\x18\x03 \x03(\x0b\x32\x12.com.kcl.api.Error"d\n\x11ParseProgram_Args\x12\r\n\x05paths\x18\x01 \x03(\t\x12\x0f\n\x07sources\x18\x02 \x03(\t\x12/\n\rexternal_pkgs\x18\x03 \x03(\x0b\x32\x18.com.kcl.api.ExternalPkg"Z\n\x13ParseProgram_Result\x12\x10\n\x08\x61st_json\x18\x01 \x01(\t\x12\r\n\x05paths\x18\x02 \x03(\t\x12"\n\x06\x65rrors\x18\x03 \x03(\x0b\x32\x12.com.kcl.api.Error"\x89\x01\n\x10LoadPackage_Args\x12\x32\n\nparse_args\x18\x01 \x01(\x0b\x32\x1e.com.kcl.api.ParseProgram_Args\x12\x13\n\x0bresolve_ast\x18\x02 \x01(\x08\x12\x14\n\x0cload_builtin\x18\x03 \x01(\x08\x12\x16\n\x0ewith_ast_index\x18\x04 \x01(\x08"\xf7\x07\n\x12LoadPackage_Result\x12\x0f\n\x07program\x18\x01 \x01(\t\x12\r\n\x05paths\x18\x02 \x03(\t\x12(\n\x0cparse_errors\x18\x03 \x03(\x0b\x32\x12.com.kcl.api.Error\x12\'\n\x0btype_errors\x18\x04 \x03(\x0b\x32\x12.com.kcl.api.Error\x12;\n\x06scopes\x18\x05 \x03(\x0b\x32+.com.kcl.api.LoadPackage_Result.ScopesEntry\x12=\n\x07symbols\x18\x06 \x03(\x0b\x32,.com.kcl.api.LoadPackage_Result.SymbolsEntry\x12K\n\x0fnode_symbol_map\x18\x07 \x03(\x0b\x32\x32.com.kcl.api.LoadPackage_Result.NodeSymbolMapEntry\x12K\n\x0fsymbol_node_map\x18\x08 \x03(\x0b\x32\x32.com.kcl.api.LoadPackage_Result.SymbolNodeMapEntry\x12\\\n\x18\x66ully_qualified_name_map\x18\t \x03(\x0b\x32:.com.kcl.api.LoadPackage_Result.FullyQualifiedNameMapEntry\x12G\n\rpkg_scope_map\x18\n \x03(\x0b\x32\x30.com.kcl.api.LoadPackage_Result.PkgScopeMapEntry\x1a\x41\n\x0bScopesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.com.kcl.api.Scope:\x02\x38\x01\x1a\x43\n\x0cSymbolsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12"\n\x05value\x18\x02 \x01(\x0b\x32\x13.com.kcl.api.Symbol:\x02\x38\x01\x1aN\n\x12NodeSymbolMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.com.kcl.api.SymbolIndex:\x02\x38\x01\x1a\x34\n\x12SymbolNodeMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aV\n\x1a\x46ullyQualifiedNameMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.com.kcl.api.SymbolIndex:\x02\x38\x01\x1aK\n\x10PkgScopeMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.com.kcl.api.ScopeIndex:\x02\x38\x01">\n\x12ListOptions_Result\x12(\n\x07options\x18\x02 \x03(\x0b\x32\x17.com.kcl.api.OptionHelp"_\n\nOptionHelp\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x10\n\x08required\x18\x03 \x01(\x08\x12\x15\n\rdefault_value\x18\x04 \x01(\t\x12\x0c\n\x04help\x18\x05 \x01(\t"\xc4\x01\n\x06Symbol\x12 \n\x02ty\x18\x01 \x01(\x0b\x32\x14.com.kcl.api.KclType\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\'\n\x05owner\x18\x03 \x01(\x0b\x32\x18.com.kcl.api.SymbolIndex\x12%\n\x03\x64\x65\x66\x18\x04 \x01(\x0b\x32\x18.com.kcl.api.SymbolIndex\x12\'\n\x05\x61ttrs\x18\x05 \x03(\x0b\x32\x18.com.kcl.api.SymbolIndex\x12\x11\n\tis_global\x18\x06 \x01(\x08"\xba\x01\n\x05Scope\x12\x0c\n\x04kind\x18\x01 \x01(\t\x12\'\n\x06parent\x18\x02 \x01(\x0b\x32\x17.com.kcl.api.ScopeIndex\x12\'\n\x05owner\x18\x03 \x01(\x0b\x32\x18.com.kcl.api.SymbolIndex\x12)\n\x08\x63hildren\x18\x04 \x03(\x0b\x32\x17.com.kcl.api.ScopeIndex\x12&\n\x04\x64\x65\x66s\x18\x05 \x03(\x0b\x32\x18.com.kcl.api.SymbolIndex"1\n\x0bSymbolIndex\x12\t\n\x01i\x18\x01 \x01(\x04\x12\t\n\x01g\x18\x02 \x01(\x04\x12\x0c\n\x04kind\x18\x03 \x01(\t"0\n\nScopeIndex\x12\t\n\x01i\x18\x01 \x01(\x04\x12\t\n\x01g\x18\x02 \x01(\x04\x12\x0c\n\x04kind\x18\x03 \x01(\t"\xd0\x03\n\x10\x45xecProgram_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\x17\n\x0fk_filename_list\x18\x02 \x03(\t\x12\x13\n\x0bk_code_list\x18\x03 \x03(\t\x12#\n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x15.com.kcl.api.Argument\x12\x11\n\toverrides\x18\x05 \x03(\t\x12\x1b\n\x13\x64isable_yaml_result\x18\x06 \x01(\x08\x12\x1a\n\x12print_override_ast\x18\x07 \x01(\x08\x12\x1a\n\x12strict_range_check\x18\x08 \x01(\x08\x12\x14\n\x0c\x64isable_none\x18\t \x01(\x08\x12\x0f\n\x07verbose\x18\n \x01(\x05\x12\r\n\x05\x64\x65\x62ug\x18\x0b \x01(\x05\x12\x11\n\tsort_keys\x18\x0c \x01(\x08\x12/\n\rexternal_pkgs\x18\r \x03(\x0b\x32\x18.com.kcl.api.ExternalPkg\x12 \n\x18include_schema_type_path\x18\x0e \x01(\x08\x12\x14\n\x0c\x63ompile_only\x18\x0f \x01(\x08\x12\x13\n\x0bshow_hidden\x18\x10 \x01(\x08\x12\x15\n\rpath_selector\x18\x11 \x03(\t\x12\x11\n\tfast_eval\x18\x12 \x01(\x08"h\n\x12\x45xecProgram_Result\x12\x13\n\x0bjson_result\x18\x01 \x01(\t\x12\x13\n\x0byaml_result\x18\x02 \x01(\t\x12\x13\n\x0blog_message\x18\x03 \x01(\t\x12\x13\n\x0b\x65rr_message\x18\x04 \x01(\t"U\n\x11\x42uildProgram_Args\x12\x30\n\texec_args\x18\x01 \x01(\x0b\x32\x1d.com.kcl.api.ExecProgram_Args\x12\x0e\n\x06output\x18\x02 \x01(\t"#\n\x13\x42uildProgram_Result\x12\x0c\n\x04path\x18\x01 \x01(\t"S\n\x11\x45xecArtifact_Args\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x30\n\texec_args\x18\x02 \x01(\x0b\x32\x1d.com.kcl.api.ExecProgram_Args"!\n\x0f\x46ormatCode_Args\x12\x0e\n\x06source\x18\x01 \x01(\t"&\n\x11\x46ormatCode_Result\x12\x11\n\tformatted\x18\x01 \x01(\x0c"\x1f\n\x0f\x46ormatPath_Args\x12\x0c\n\x04path\x18\x01 \x01(\t"*\n\x11\x46ormatPath_Result\x12\x15\n\rchanged_paths\x18\x01 \x03(\t"\x1e\n\rLintPath_Args\x12\r\n\x05paths\x18\x01 \x03(\t""\n\x0fLintPath_Result\x12\x0f\n\x07results\x18\x01 \x03(\t"F\n\x11OverrideFile_Args\x12\x0c\n\x04\x66ile\x18\x01 \x01(\t\x12\r\n\x05specs\x18\x02 \x03(\t\x12\x14\n\x0cimport_paths\x18\x03 \x03(\t"O\n\x13OverrideFile_Result\x12\x0e\n\x06result\x18\x01 \x01(\x08\x12(\n\x0cparse_errors\x18\x02 \x03(\x0b\x32\x12.com.kcl.api.Error".\n\x15ListVariables_Options\x12\x15\n\rmerge_program\x18\x01 \x01(\x08"8\n\x0cVariableList\x12(\n\tvariables\x18\x01 \x03(\x0b\x32\x15.com.kcl.api.Variable"g\n\x12ListVariables_Args\x12\r\n\x05\x66iles\x18\x01 \x03(\t\x12\r\n\x05specs\x18\x02 \x03(\t\x12\x33\n\x07options\x18\x03 \x01(\x0b\x32".com.kcl.api.ListVariables_Options"\xed\x01\n\x14ListVariables_Result\x12\x43\n\tvariables\x18\x01 \x03(\x0b\x32\x30.com.kcl.api.ListVariables_Result.VariablesEntry\x12\x19\n\x11unsupported_codes\x18\x02 \x03(\t\x12(\n\x0cparse_errors\x18\x03 \x03(\x0b\x32\x12.com.kcl.api.Error\x1aK\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.com.kcl.api.VariableList:\x02\x38\x01"\x94\x01\n\x08Variable\x12\r\n\x05value\x18\x01 \x01(\t\x12\x11\n\ttype_name\x18\x02 \x01(\t\x12\x0e\n\x06op_sym\x18\x03 \x01(\t\x12)\n\nlist_items\x18\x04 \x03(\x0b\x32\x15.com.kcl.api.Variable\x12+\n\x0c\x64ict_entries\x18\x05 \x03(\x0b\x32\x15.com.kcl.api.MapEntry"=\n\x08MapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.com.kcl.api.Variable"b\n\x19GetSchemaTypeMapping_Args\x12\x30\n\texec_args\x18\x01 \x01(\x0b\x32\x1d.com.kcl.api.ExecProgram_Args\x12\x13\n\x0bschema_name\x18\x02 \x01(\t"\xcb\x01\n\x1bGetSchemaTypeMapping_Result\x12\\\n\x13schema_type_mapping\x18\x01 \x03(\x0b\x32?.com.kcl.api.GetSchemaTypeMapping_Result.SchemaTypeMappingEntry\x1aN\n\x16SchemaTypeMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.com.kcl.api.KclType:\x02\x38\x01"\x87\x01\n\x11ValidateCode_Args\x12\x10\n\x08\x64\x61tafile\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\x12\x0c\n\x04\x66ile\x18\x03 \x01(\t\x12\x0c\n\x04\x63ode\x18\x04 \x01(\t\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x16\n\x0e\x61ttribute_name\x18\x06 \x01(\t\x12\x0e\n\x06\x66ormat\x18\x07 \x01(\t";\n\x13ValidateCode_Result\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x13\n\x0b\x65rr_message\x18\x02 \x01(\t":\n\x08Position\x12\x0c\n\x04line\x18\x01 \x01(\x03\x12\x0e\n\x06\x63olumn\x18\x02 \x01(\x03\x12\x10\n\x08\x66ilename\x18\x03 \x01(\t"i\n\x11ListDepFiles_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\x14\n\x0cuse_abs_path\x18\x02 \x01(\x08\x12\x13\n\x0binclude_all\x18\x03 \x01(\x08\x12\x17\n\x0fuse_fast_parser\x18\x04 \x01(\x08"F\n\x13ListDepFiles_Result\x12\x0f\n\x07pkgroot\x18\x01 \x01(\t\x12\x0f\n\x07pkgpath\x18\x02 \x01(\t\x12\r\n\x05\x66iles\x18\x03 \x03(\t"9\n\x16LoadSettingsFiles_Args\x12\x10\n\x08work_dir\x18\x01 \x01(\t\x12\r\n\x05\x66iles\x18\x02 \x03(\t"{\n\x18LoadSettingsFiles_Result\x12/\n\x0fkcl_cli_configs\x18\x01 \x01(\x0b\x32\x16.com.kcl.api.CliConfig\x12.\n\x0bkcl_options\x18\x02 \x03(\x0b\x32\x19.com.kcl.api.KeyValuePair"\x83\x02\n\tCliConfig\x12\r\n\x05\x66iles\x18\x01 \x03(\t\x12\x0e\n\x06output\x18\x02 \x01(\t\x12\x11\n\toverrides\x18\x03 \x03(\t\x12\x15\n\rpath_selector\x18\x04 \x03(\t\x12\x1a\n\x12strict_range_check\x18\x05 \x01(\x08\x12\x14\n\x0c\x64isable_none\x18\x06 \x01(\x08\x12\x0f\n\x07verbose\x18\x07 \x01(\x03\x12\r\n\x05\x64\x65\x62ug\x18\x08 \x01(\x08\x12\x11\n\tsort_keys\x18\t \x01(\x08\x12\x13\n\x0bshow_hidden\x18\n \x01(\x08\x12 \n\x18include_schema_type_path\x18\x0b \x01(\x08\x12\x11\n\tfast_eval\x18\x0c \x01(\x08"*\n\x0cKeyValuePair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"^\n\x0bRename_Args\x12\x14\n\x0cpackage_root\x18\x01 \x01(\t\x12\x13\n\x0bsymbol_path\x18\x02 \x01(\t\x12\x12\n\nfile_paths\x18\x03 \x03(\t\x12\x10\n\x08new_name\x18\x04 \x01(\t"&\n\rRename_Result\x12\x15\n\rchanged_files\x18\x01 \x03(\t"\xc7\x01\n\x0fRenameCode_Args\x12\x14\n\x0cpackage_root\x18\x01 \x01(\t\x12\x13\n\x0bsymbol_path\x18\x02 \x01(\t\x12\x43\n\x0csource_codes\x18\x03 \x03(\x0b\x32-.com.kcl.api.RenameCode_Args.SourceCodesEntry\x12\x10\n\x08new_name\x18\x04 \x01(\t\x1a\x32\n\x10SourceCodesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"\x91\x01\n\x11RenameCode_Result\x12G\n\rchanged_codes\x18\x01 \x03(\x0b\x32\x30.com.kcl.api.RenameCode_Result.ChangedCodesEntry\x1a\x33\n\x11\x43hangedCodesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"v\n\tTest_Args\x12\x30\n\texec_args\x18\x01 \x01(\x0b\x32\x1d.com.kcl.api.ExecProgram_Args\x12\x10\n\x08pkg_list\x18\x02 \x03(\t\x12\x12\n\nrun_regexp\x18\x03 \x01(\t\x12\x11\n\tfail_fast\x18\x04 \x01(\x08"6\n\x0bTest_Result\x12\'\n\x04info\x18\x02 \x03(\x0b\x32\x19.com.kcl.api.TestCaseInfo"R\n\x0cTestCaseInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x10\n\x08\x64uration\x18\x03 \x01(\x04\x12\x13\n\x0blog_message\x18\x04 \x01(\t"@\n\x17UpdateDependencies_Args\x12\x15\n\rmanifest_path\x18\x01 \x01(\t\x12\x0e\n\x06vendor\x18\x02 \x01(\x08"L\n\x19UpdateDependencies_Result\x12/\n\rexternal_pkgs\x18\x03 \x03(\x0b\x32\x18.com.kcl.api.ExternalPkg"\xf3\x04\n\x07KclType\x12\x0c\n\x04type\x18\x01 \x01(\t\x12)\n\x0bunion_types\x18\x02 \x03(\x0b\x32\x14.com.kcl.api.KclType\x12\x0f\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\t\x12\x13\n\x0bschema_name\x18\x04 \x01(\t\x12\x12\n\nschema_doc\x18\x05 \x01(\t\x12\x38\n\nproperties\x18\x06 \x03(\x0b\x32$.com.kcl.api.KclType.PropertiesEntry\x12\x10\n\x08required\x18\x07 \x03(\t\x12!\n\x03key\x18\x08 \x01(\x0b\x32\x14.com.kcl.api.KclType\x12"\n\x04item\x18\t \x01(\x0b\x32\x14.com.kcl.api.KclType\x12\x0c\n\x04line\x18\n \x01(\x05\x12*\n\ndecorators\x18\x0b \x03(\x0b\x32\x16.com.kcl.api.Decorator\x12\x10\n\x08\x66ilename\x18\x0c \x01(\t\x12\x10\n\x08pkg_path\x18\r \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x0e \x01(\t\x12\x34\n\x08\x65xamples\x18\x0f \x03(\x0b\x32".com.kcl.api.KclType.ExamplesEntry\x12)\n\x0b\x62\x61se_schema\x18\x10 \x01(\x0b\x32\x14.com.kcl.api.KclType\x1aG\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.com.kcl.api.KclType:\x02\x38\x01\x1a\x45\n\rExamplesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.com.kcl.api.Example:\x02\x38\x01"\x95\x01\n\tDecorator\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x03(\t\x12\x36\n\x08keywords\x18\x03 \x03(\x0b\x32$.com.kcl.api.Decorator.KeywordsEntry\x1a/\n\rKeywordsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01">\n\x07\x45xample\x12\x0f\n\x07summary\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t2\x96\x01\n\x0e\x42uiltinService\x12\x38\n\x04Ping\x12\x16.com.kcl.api.Ping_Args\x1a\x18.com.kcl.api.Ping_Result\x12J\n\nListMethod\x12\x1c.com.kcl.api.ListMethod_Args\x1a\x1e.com.kcl.api.ListMethod_Result2\xde\r\n\x0cKclvmService\x12\x38\n\x04Ping\x12\x16.com.kcl.api.Ping_Args\x1a\x18.com.kcl.api.Ping_Result\x12J\n\nGetVersion\x12\x1c.com.kcl.api.GetVersion_Args\x1a\x1e.com.kcl.api.GetVersion_Result\x12P\n\x0cParseProgram\x12\x1e.com.kcl.api.ParseProgram_Args\x1a .com.kcl.api.ParseProgram_Result\x12G\n\tParseFile\x12\x1b.com.kcl.api.ParseFile_Args\x1a\x1d.com.kcl.api.ParseFile_Result\x12M\n\x0bLoadPackage\x12\x1d.com.kcl.api.LoadPackage_Args\x1a\x1f.com.kcl.api.LoadPackage_Result\x12N\n\x0bListOptions\x12\x1e.com.kcl.api.ParseProgram_Args\x1a\x1f.com.kcl.api.ListOptions_Result\x12S\n\rListVariables\x12\x1f.com.kcl.api.ListVariables_Args\x1a!.com.kcl.api.ListVariables_Result\x12M\n\x0b\x45xecProgram\x12\x1d.com.kcl.api.ExecProgram_Args\x1a\x1f.com.kcl.api.ExecProgram_Result\x12P\n\x0c\x42uildProgram\x12\x1e.com.kcl.api.BuildProgram_Args\x1a .com.kcl.api.BuildProgram_Result\x12O\n\x0c\x45xecArtifact\x12\x1e.com.kcl.api.ExecArtifact_Args\x1a\x1f.com.kcl.api.ExecProgram_Result\x12P\n\x0cOverrideFile\x12\x1e.com.kcl.api.OverrideFile_Args\x1a .com.kcl.api.OverrideFile_Result\x12h\n\x14GetSchemaTypeMapping\x12&.com.kcl.api.GetSchemaTypeMapping_Args\x1a(.com.kcl.api.GetSchemaTypeMapping_Result\x12J\n\nFormatCode\x12\x1c.com.kcl.api.FormatCode_Args\x1a\x1e.com.kcl.api.FormatCode_Result\x12J\n\nFormatPath\x12\x1c.com.kcl.api.FormatPath_Args\x1a\x1e.com.kcl.api.FormatPath_Result\x12\x44\n\x08LintPath\x12\x1a.com.kcl.api.LintPath_Args\x1a\x1c.com.kcl.api.LintPath_Result\x12P\n\x0cValidateCode\x12\x1e.com.kcl.api.ValidateCode_Args\x1a .com.kcl.api.ValidateCode_Result\x12P\n\x0cListDepFiles\x12\x1e.com.kcl.api.ListDepFiles_Args\x1a .com.kcl.api.ListDepFiles_Result\x12_\n\x11LoadSettingsFiles\x12#.com.kcl.api.LoadSettingsFiles_Args\x1a%.com.kcl.api.LoadSettingsFiles_Result\x12>\n\x06Rename\x12\x18.com.kcl.api.Rename_Args\x1a\x1a.com.kcl.api.Rename_Result\x12J\n\nRenameCode\x12\x1c.com.kcl.api.RenameCode_Args\x1a\x1e.com.kcl.api.RenameCode_Result\x12\x38\n\x04Test\x12\x16.com.kcl.api.Test_Args\x1a\x18.com.kcl.api.Test_Result\x12\x62\n\x12UpdateDependencies\x12$.com.kcl.api.UpdateDependencies_Args\x1a&.com.kcl.api.UpdateDependencies_ResultB\x14Z\x05.;lib\xaa\x02\nKclLib.APIb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "spec_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    _globals["DESCRIPTOR"]._loaded_options = None
    _globals["DESCRIPTOR"]._serialized_options = b"Z\005.;lib\252\002\nKclLib.API"
    _globals["_LOADPACKAGE_RESULT_SCOPESENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_SCOPESENTRY"]._serialized_options = b"8\001"
    _globals["_LOADPACKAGE_RESULT_SYMBOLSENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_SYMBOLSENTRY"]._serialized_options = b"8\001"
    _globals["_LOADPACKAGE_RESULT_NODESYMBOLMAPENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_NODESYMBOLMAPENTRY"]._serialized_options = b"8\001"
    _globals["_LOADPACKAGE_RESULT_SYMBOLNODEMAPENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_SYMBOLNODEMAPENTRY"]._serialized_options = b"8\001"
    _globals["_LOADPACKAGE_RESULT_FULLYQUALIFIEDNAMEMAPENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_FULLYQUALIFIEDNAMEMAPENTRY"]._serialized_options = (
        b"8\001"
    )
    _globals["_LOADPACKAGE_RESULT_PKGSCOPEMAPENTRY"]._loaded_options = None
    _globals["_LOADPACKAGE_RESULT_PKGSCOPEMAPENTRY"]._serialized_options = b"8\001"
    _globals["_LISTVARIABLES_RESULT_VARIABLESENTRY"]._loaded_options = None
    _globals["_LISTVARIABLES_RESULT_VARIABLESENTRY"]._serialized_options = b"8\001"
    _globals["_GETSCHEMATYPEMAPPING_RESULT_SCHEMATYPEMAPPINGENTRY"]._loaded_options = (
        None
    )
    _globals[
        "_GETSCHEMATYPEMAPPING_RESULT_SCHEMATYPEMAPPINGENTRY"
    ]._serialized_options = b"8\001"
    _globals["_RENAMECODE_ARGS_SOURCECODESENTRY"]._loaded_options = None
    _globals["_RENAMECODE_ARGS_SOURCECODESENTRY"]._serialized_options = b"8\001"
    _globals["_RENAMECODE_RESULT_CHANGEDCODESENTRY"]._loaded_options = None
    _globals["_RENAMECODE_RESULT_CHANGEDCODESENTRY"]._serialized_options = b"8\001"
    _globals["_KCLTYPE_PROPERTIESENTRY"]._loaded_options = None
    _globals["_KCLTYPE_PROPERTIESENTRY"]._serialized_options = b"8\001"
    _globals["_KCLTYPE_EXAMPLESENTRY"]._loaded_options = None
    _globals["_KCLTYPE_EXAMPLESENTRY"]._serialized_options = b"8\001"
    _globals["_DECORATOR_KEYWORDSENTRY"]._loaded_options = None
    _globals["_DECORATOR_KEYWORDSENTRY"]._serialized_options = b"8\001"
    _globals["_EXTERNALPKG"]._serialized_start = 27
    _globals["_EXTERNALPKG"]._serialized_end = 76
    _globals["_ARGUMENT"]._serialized_start = 78
    _globals["_ARGUMENT"]._serialized_end = 117
    _globals["_ERROR"]._serialized_start = 119
    _globals["_ERROR"]._serialized_end = 195
    _globals["_MESSAGE"]._serialized_start = 197
    _globals["_MESSAGE"]._serialized_end = 255
    _globals["_PING_ARGS"]._serialized_start = 257
    _globals["_PING_ARGS"]._serialized_end = 283
    _globals["_PING_RESULT"]._serialized_start = 285
    _globals["_PING_RESULT"]._serialized_end = 313
    _globals["_GETVERSION_ARGS"]._serialized_start = 315
    _globals["_GETVERSION_ARGS"]._serialized_end = 332
    _globals["_GETVERSION_RESULT"]._serialized_start = 334
    _globals["_GETVERSION_RESULT"]._serialized_end = 427
    _globals["_LISTMETHOD_ARGS"]._serialized_start = 429
    _globals["_LISTMETHOD_ARGS"]._serialized_end = 446
    _globals["_LISTMETHOD_RESULT"]._serialized_start = 448
    _globals["_LISTMETHOD_RESULT"]._serialized_end = 493
    _globals["_PARSEFILE_ARGS"]._serialized_start = 495
    _globals["_PARSEFILE_ARGS"]._serialized_end = 590
    _globals["_PARSEFILE_RESULT"]._serialized_start = 592
    _globals["_PARSEFILE_RESULT"]._serialized_end = 678
    _globals["_PARSEPROGRAM_ARGS"]._serialized_start = 680
    _globals["_PARSEPROGRAM_ARGS"]._serialized_end = 780
    _globals["_PARSEPROGRAM_RESULT"]._serialized_start = 782
    _globals["_PARSEPROGRAM_RESULT"]._serialized_end = 872
    _globals["_LOADPACKAGE_ARGS"]._serialized_start = 875
    _globals["_LOADPACKAGE_ARGS"]._serialized_end = 1012
    _globals["_LOADPACKAGE_RESULT"]._serialized_start = 1015
    _globals["_LOADPACKAGE_RESULT"]._serialized_end = 2030
    _globals["_LOADPACKAGE_RESULT_SCOPESENTRY"]._serialized_start = 1597
    _globals["_LOADPACKAGE_RESULT_SCOPESENTRY"]._serialized_end = 1662
    _globals["_LOADPACKAGE_RESULT_SYMBOLSENTRY"]._serialized_start = 1664
    _globals["_LOADPACKAGE_RESULT_SYMBOLSENTRY"]._serialized_end = 1731
    _globals["_LOADPACKAGE_RESULT_NODESYMBOLMAPENTRY"]._serialized_start = 1733
    _globals["_LOADPACKAGE_RESULT_NODESYMBOLMAPENTRY"]._serialized_end = 1811
    _globals["_LOADPACKAGE_RESULT_SYMBOLNODEMAPENTRY"]._serialized_start = 1813
    _globals["_LOADPACKAGE_RESULT_SYMBOLNODEMAPENTRY"]._serialized_end = 1865
    _globals["_LOADPACKAGE_RESULT_FULLYQUALIFIEDNAMEMAPENTRY"]._serialized_start = 1867
    _globals["_LOADPACKAGE_RESULT_FULLYQUALIFIEDNAMEMAPENTRY"]._serialized_end = 1953
    _globals["_LOADPACKAGE_RESULT_PKGSCOPEMAPENTRY"]._serialized_start = 1955
    _globals["_LOADPACKAGE_RESULT_PKGSCOPEMAPENTRY"]._serialized_end = 2030
    _globals["_LISTOPTIONS_RESULT"]._serialized_start = 2032
    _globals["_LISTOPTIONS_RESULT"]._serialized_end = 2094
    _globals["_OPTIONHELP"]._serialized_start = 2096
    _globals["_OPTIONHELP"]._serialized_end = 2191
    _globals["_SYMBOL"]._serialized_start = 2194
    _globals["_SYMBOL"]._serialized_end = 2390
    _globals["_SCOPE"]._serialized_start = 2393
    _globals["_SCOPE"]._serialized_end = 2579
    _globals["_SYMBOLINDEX"]._serialized_start = 2581
    _globals["_SYMBOLINDEX"]._serialized_end = 2630
    _globals["_SCOPEINDEX"]._serialized_start = 2632
    _globals["_SCOPEINDEX"]._serialized_end = 2680
    _globals["_EXECPROGRAM_ARGS"]._serialized_start = 2683
    _globals["_EXECPROGRAM_ARGS"]._serialized_end = 3147
    _globals["_EXECPROGRAM_RESULT"]._serialized_start = 3149
    _globals["_EXECPROGRAM_RESULT"]._serialized_end = 3253
    _globals["_BUILDPROGRAM_ARGS"]._serialized_start = 3255
    _globals["_BUILDPROGRAM_ARGS"]._serialized_end = 3340
    _globals["_BUILDPROGRAM_RESULT"]._serialized_start = 3342
    _globals["_BUILDPROGRAM_RESULT"]._serialized_end = 3377
    _globals["_EXECARTIFACT_ARGS"]._serialized_start = 3379
    _globals["_EXECARTIFACT_ARGS"]._serialized_end = 3462
    _globals["_FORMATCODE_ARGS"]._serialized_start = 3464
    _globals["_FORMATCODE_ARGS"]._serialized_end = 3497
    _globals["_FORMATCODE_RESULT"]._serialized_start = 3499
    _globals["_FORMATCODE_RESULT"]._serialized_end = 3537
    _globals["_FORMATPATH_ARGS"]._serialized_start = 3539
    _globals["_FORMATPATH_ARGS"]._serialized_end = 3570
    _globals["_FORMATPATH_RESULT"]._serialized_start = 3572
    _globals["_FORMATPATH_RESULT"]._serialized_end = 3614
    _globals["_LINTPATH_ARGS"]._serialized_start = 3616
    _globals["_LINTPATH_ARGS"]._serialized_end = 3646
    _globals["_LINTPATH_RESULT"]._serialized_start = 3648
    _globals["_LINTPATH_RESULT"]._serialized_end = 3682
    _globals["_OVERRIDEFILE_ARGS"]._serialized_start = 3684
    _globals["_OVERRIDEFILE_ARGS"]._serialized_end = 3754
    _globals["_OVERRIDEFILE_RESULT"]._serialized_start = 3756
    _globals["_OVERRIDEFILE_RESULT"]._serialized_end = 3835
    _globals["_LISTVARIABLES_OPTIONS"]._serialized_start = 3837
    _globals["_LISTVARIABLES_OPTIONS"]._serialized_end = 3883
    _globals["_VARIABLELIST"]._serialized_start = 3885
    _globals["_VARIABLELIST"]._serialized_end = 3941
    _globals["_LISTVARIABLES_ARGS"]._serialized_start = 3943
    _globals["_LISTVARIABLES_ARGS"]._serialized_end = 4046
    _globals["_LISTVARIABLES_RESULT"]._serialized_start = 4049
    _globals["_LISTVARIABLES_RESULT"]._serialized_end = 4286
    _globals["_LISTVARIABLES_RESULT_VARIABLESENTRY"]._serialized_start = 4211
    _globals["_LISTVARIABLES_RESULT_VARIABLESENTRY"]._serialized_end = 4286
    _globals["_VARIABLE"]._serialized_start = 4289
    _globals["_VARIABLE"]._serialized_end = 4437
    _globals["_MAPENTRY"]._serialized_start = 4439
    _globals["_MAPENTRY"]._serialized_end = 4500
    _globals["_GETSCHEMATYPEMAPPING_ARGS"]._serialized_start = 4502
    _globals["_GETSCHEMATYPEMAPPING_ARGS"]._serialized_end = 4600
    _globals["_GETSCHEMATYPEMAPPING_RESULT"]._serialized_start = 4603
    _globals["_GETSCHEMATYPEMAPPING_RESULT"]._serialized_end = 4806
    _globals[
        "_GETSCHEMATYPEMAPPING_RESULT_SCHEMATYPEMAPPINGENTRY"
    ]._serialized_start = 4728
    _globals["_GETSCHEMATYPEMAPPING_RESULT_SCHEMATYPEMAPPINGENTRY"]._serialized_end = (
        4806
    )
    _globals["_VALIDATECODE_ARGS"]._serialized_start = 4809
    _globals["_VALIDATECODE_ARGS"]._serialized_end = 4944
    _globals["_VALIDATECODE_RESULT"]._serialized_start = 4946
    _globals["_VALIDATECODE_RESULT"]._serialized_end = 5005
    _globals["_POSITION"]._serialized_start = 5007
    _globals["_POSITION"]._serialized_end = 5065
    _globals["_LISTDEPFILES_ARGS"]._serialized_start = 5067
    _globals["_LISTDEPFILES_ARGS"]._serialized_end = 5172
    _globals["_LISTDEPFILES_RESULT"]._serialized_start = 5174
    _globals["_LISTDEPFILES_RESULT"]._serialized_end = 5244
    _globals["_LOADSETTINGSFILES_ARGS"]._serialized_start = 5246
    _globals["_LOADSETTINGSFILES_ARGS"]._serialized_end = 5303
    _globals["_LOADSETTINGSFILES_RESULT"]._serialized_start = 5305
    _globals["_LOADSETTINGSFILES_RESULT"]._serialized_end = 5428
    _globals["_CLICONFIG"]._serialized_start = 5431
    _globals["_CLICONFIG"]._serialized_end = 5690
    _globals["_KEYVALUEPAIR"]._serialized_start = 5692
    _globals["_KEYVALUEPAIR"]._serialized_end = 5734
    _globals["_RENAME_ARGS"]._serialized_start = 5736
    _globals["_RENAME_ARGS"]._serialized_end = 5830
    _globals["_RENAME_RESULT"]._serialized_start = 5832
    _globals["_RENAME_RESULT"]._serialized_end = 5870
    _globals["_RENAMECODE_ARGS"]._serialized_start = 5873
    _globals["_RENAMECODE_ARGS"]._serialized_end = 6072
    _globals["_RENAMECODE_ARGS_SOURCECODESENTRY"]._serialized_start = 6022
    _globals["_RENAMECODE_ARGS_SOURCECODESENTRY"]._serialized_end = 6072
    _globals["_RENAMECODE_RESULT"]._serialized_start = 6075
    _globals["_RENAMECODE_RESULT"]._serialized_end = 6220
    _globals["_RENAMECODE_RESULT_CHANGEDCODESENTRY"]._serialized_start = 6169
    _globals["_RENAMECODE_RESULT_CHANGEDCODESENTRY"]._serialized_end = 6220
    _globals["_TEST_ARGS"]._serialized_start = 6222
    _globals["_TEST_ARGS"]._serialized_end = 6340
    _globals["_TEST_RESULT"]._serialized_start = 6342
    _globals["_TEST_RESULT"]._serialized_end = 6396
    _globals["_TESTCASEINFO"]._serialized_start = 6398
    _globals["_TESTCASEINFO"]._serialized_end = 6480
    _globals["_UPDATEDEPENDENCIES_ARGS"]._serialized_start = 6482
    _globals["_UPDATEDEPENDENCIES_ARGS"]._serialized_end = 6546
    _globals["_UPDATEDEPENDENCIES_RESULT"]._serialized_start = 6548
    _globals["_UPDATEDEPENDENCIES_RESULT"]._serialized_end = 6624
    _globals["_KCLTYPE"]._serialized_start = 6627
    _globals["_KCLTYPE"]._serialized_end = 7254
    _globals["_KCLTYPE_PROPERTIESENTRY"]._serialized_start = 7112
    _globals["_KCLTYPE_PROPERTIESENTRY"]._serialized_end = 7183
    _globals["_KCLTYPE_EXAMPLESENTRY"]._serialized_start = 7185
    _globals["_KCLTYPE_EXAMPLESENTRY"]._serialized_end = 7254
    _globals["_DECORATOR"]._serialized_start = 7257
    _globals["_DECORATOR"]._serialized_end = 7406
    _globals["_DECORATOR_KEYWORDSENTRY"]._serialized_start = 7359
    _globals["_DECORATOR_KEYWORDSENTRY"]._serialized_end = 7406
    _globals["_EXAMPLE"]._serialized_start = 7408
    _globals["_EXAMPLE"]._serialized_end = 7470
    _globals["_BUILTINSERVICE"]._serialized_start = 7473
    _globals["_BUILTINSERVICE"]._serialized_end = 7623
    _globals["_KCLVMSERVICE"]._serialized_start = 7626
    _globals["_KCLVMSERVICE"]._serialized_end = 9384
# @@protoc_insertion_point(module_scope)



import kcl_lib
import kcl_lib.plugin as plugin
from .spec_pb2 import (
    Ping_Args,
    Ping_Result,
    GetVersion_Args,
    GetVersion_Result,
    ParseFile_Args,
    ParseFile_Result,
    ParseProgram_Args,
    ParseProgram_Result,
    LoadPackage_Args,
    LoadPackage_Result,
    ListOptions_Result,
    ExecProgram_Args,
    ExecProgram_Result,
    BuildProgram_Args,
    BuildProgram_Result,
    ExecArtifact_Args,
    FormatCode_Args,
    FormatCode_Result,
    FormatPath_Args,
    FormatPath_Result,
    LintPath_Args,
    LintPath_Result,
    OverrideFile_Args,
    OverrideFile_Result,
    ListVariables_Args,
    ListVariables_Result,
    GetSchemaTypeMapping_Args,
    GetSchemaTypeMapping_Result,
    ValidateCode_Args,
    ValidateCode_Result,
    ListDepFiles_Args,
    ListDepFiles_Result,
    LoadSettingsFiles_Args,
    LoadSettingsFiles_Result,
    Rename_Args,
    Rename_Result,
    RenameCode_Args,
    RenameCode_Result,
    Test_Args,
    Test_Result,
    UpdateDependencies_Args,
    UpdateDependencies_Result,
)
from google.protobuf import message as _message


class API:
    """KCL APIs

    ## Examples

    Python Code

    ```python
    import kcl_lib.api as api
    args = api.ExecProgram_Args(k_filename_list=["a.k"])
    api = api.API()

    result = api.exec_program(args)
    print(result.yaml_result)
    ```
    """

    def __init__(self, plugin_agent: int = plugin.plugin_agent_addr):
        self.plugin_agent = plugin_agent

    def ping(self, args: Ping_Args) -> Ping_Result:
        return self.call("KclvmService.Ping", args)

    def parse_program(self, args: ParseProgram_Args) -> ParseProgram_Result:
        """Parse KCL program with entry files and return the AST JSON string.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
           replicas: int
        app: AppConfig {
           replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ParseProgram_Args(paths=["schema.k"])
        api = api.API()
        result = api.parse_program(args)
        assert len(result.paths) == 1
        assert len(result.errors) == 0
        ```
        """
        return self.call("KclvmService.ParseProgram", args)

    def exec_program(self, args: ExecProgram_Args) -> ExecProgram_Result:
        """Execute KCL file with arguments and return the JSON/YAML result.

        ## Examples

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ExecProgram_Args(k_filename_list=["schema.k"])
        api = api.API()
        result = api.exec_program(args)
        ```

        A case with the file not found error

        ```python
        import kcl_lib.api as api
        try:
            args = api.ExecProgram_Args(k_filename_list=["file_not_found"])
            api = api.API()
            result = api.exec_program(args)
            assert False
        except Exception as err:
            assert "Cannot find the kcl file" in str(err)
        ```
        """
        return self.call("KclvmService.ExecProgram", args)

    def parse_file(self, args: ParseFile_Args) -> ParseFile_Result:
        """Parse KCL single file to Module AST JSON string with import dependencies and parse errors.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ParseProgram_Args(paths=[TEST_FILE])
        api = api.API()
        result = api.parse_program(args)
        assert len(result.paths) == 1
        assert len(result.errors) == 0
        ```
        """
        return self.call("KclvmService.ParseFile", args)

    def parse_program(self, args: ParseProgram_Args) -> ParseProgram_Result:
        """Parse KCL program with entry files and return the AST JSON string.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ParseProgram_Args(paths=["schema.k"])
        api = api.API()
        result = api.parse_program(args)
        assert len(result.paths) == 1
        assert len(result.errors) == 0
        ```
        """
        return self.call("KclvmService.ParseProgram", args)

    def load_package(self, args: LoadPackage_Args) -> LoadPackage_Result:
        """load_package provides users with the ability to parse KCL program and semantic model information including symbols, types, definitions, etc.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.LoadPackage_Args(
            parse_args=api.ParseProgram_Args(paths=["schema.k"]), resolve_ast=True
        )
        api = api.API()
        result = api.load_package(args)
        assert list(result.symbols.values())[0].ty.schema_name == "AppConfig"
        ```
        """
        return self.call("KclvmService.LoadPackage", args)

    def list_options(self, args: ParseProgram_Args) -> ListOptions_Result:
        """list_options provides users with the ability to parse KCL program and get all option information.

        ## Example

        The content of `options.k` is

        ```python
        a = option("key1")
        b = option("key2", required=True)
        c = {
            metadata.key = option("metadata-key")
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ParseProgram_Args(paths=["options.k"])
        api = api.API()
        result = api.list_options(args)
        assert len(result.options) == 3
        assert result.options[0].name == "key1"
        assert result.options[1].name == "key2"
        assert result.options[2].name == "metadata-key"
        ```
        """
        return self.call("KclvmService.ListOptions", args)

    def list_variables(self, args: ListVariables_Args) -> ListVariables_Result:
        """list_variables provides users with the ability to parse KCL program and get all variables by specs.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.ListVariables_Args(files=[TEST_FILE])
        api = api.API()
        result = api.list_variables(args)
        ```
        """
        return self.call("KclvmService.ListVariables", args)

    def format_code(self, args: FormatCode_Args) -> FormatCode_Result:
        """Format the code source.

        ## Example

        Python Code

        ```python
        import kcl_lib.api as api
        source_code = "a   =   1"
        args = api.FormatCode_Args(source=source_code)
        api_instance = api.API()

        result = api_instance.format_code(args)
        assert result.formatted.decode(), "a = 1"
        ```
        """
        return self.call("KclvmService.FormatCode", args)

    def format_path(self, args: FormatPath_Args) -> FormatPath_Result:
        """Format KCL file or directory path contains KCL files and returns the changed file paths.

        ## Example

        The content of `format_path.k` is

        ```python
        schema Person:
            name:   str
            age:    int
            check:
                0 <   age <   120
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.FormatPath_Args(path="format_path.k")
        api_instance = api.API()
        result = api_instance.format_path(args)
        print(result)
        ```
        """
        return self.call("KclvmService.FormatPath", args)

    def lint_path(self, args: LintPath_Args) -> LintPath_Result:
        """Lint files and return error messages including errors and warnings.

        ## Example

        The content of `lint_path.k` is

        ```python
        import math

        ```

        Python Code

        ```python
        import kcl_lib.api as api

        args = api.LintPath_Args(paths=["lint_path.k"])
        api_instance = api.API()
        result = api_instance.lint_path(args)
        ```
        """
        return self.call("KclvmService.LintPath", args)

    def override_file(self, args: OverrideFile_Args) -> OverrideFile_Result:
        """Override KCL file with arguments. See [https://www.kcl-lang.io/docs/user_docs/guides/automation](https://www.kcl-lang.io/docs/user_docs/guides/automation) for more override spec guide.

        ## Example

        The content of `main.k` is

        ```python
        a = 1
        b = {
            "a": 1
            "b": 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        import pathlib

        test_file = "main.k"
        args = api.OverrideFile_Args(
            file=test_file,
            specs=["b.a=2"],
        )
        api = api.API()
        result = api.override_file(args)
        assert len(result.parse_errors) == 0
        assert result.result == True
        assert pathlib.Path(test_file).read_text() == \"\"\"a = 1
        b = {
            "a": 2
            "b": 2
        }
        \"\"\"
        ```
        """
        return self.call("KclvmService.OverrideFile", args)

    def get_schema_type_mapping(
        self,
        args: GetSchemaTypeMapping_Args,
    ) -> GetSchemaTypeMapping_Result:
        """Get schema type mapping defined in the program.

        ## Example

        The content of `schema.k` is

        ```python
        schema AppConfig:
            replicas: int
        app: AppConfig {
            replicas: 2
        }
        ```

        Python Code

        ```python
        import kcl_lib.api as api

        exec_args = api.ExecProgram_Args(k_filename_list=["schema.k"])
        args = api.GetSchemaTypeMapping_Args(exec_args=exec_args)
        api = api.API()
        result = api.get_schema_type_mapping(args)
        assert result.schema_type_mapping["app"].properties["replicas"].type == "int"
        ```
        """
        return self.call("KclvmService.GetSchemaTypeMapping", args)

    def validate_code(self, args: ValidateCode_Args) -> ValidateCode_Result:
        """Validate code using schema and JSON/YAML data strings.

        ## Example
        Python Code

        ```python
        import kcl_lib.api as api

        code = \"\"\"
        schema Person:
            name: str
            age: int
            check:
                0 < age < 120
        \"\"\"
        data = '{"name": "Alice", "age": 10}'
        args = api.ValidateCode_Args(code=code, data=data, format="json")
        api_instance = api.API()
        result = api_instance.validate_code(args)
        assert result.success == True
        assert result.err_message == ""
        ```
        """
        return self.call("KclvmService.ValidateCode", args)

    def load_settings_files(
        self,
        args: LoadSettingsFiles_Args,
    ) -> LoadSettingsFiles_Result:
        """Load the setting file config defined in `kcl.yaml`

        ## Example

        The content of `kcl.yaml` is

        ```yaml
        kcl_cli_configs:
          strict_range_check: true
        kcl_options:
          - key: key
            value: value
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.LoadSettingsFiles_Args(
            work_dir=".", files=["kcl.yaml"]
        )
        api_instance = api.API()
        result = api_instance.load_settings_files(args)
        assert result.kcl_cli_configs.files == []
        assert result.kcl_cli_configs.strict_range_check == True
        assert (
            result.kcl_options[0].key == "key" and result.kcl_options[0].value == '"value"'
        )
        ```
        """
        return self.call("KclvmService.LoadSettingsFiles", args)

    def rename(self, args: Rename_Args) -> Rename_Result:
        """Rename all the occurrences of the target symbol in the files. This API will rewrite files if they contain symbols to be renamed. Return the file paths that got changed.

        ## Example

        The content of main.k is

        ```python

        a = 1
        b = a
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.Rename_Args(
            package_root=".",
            symbol_path="a",
            file_paths=["main.k"],
            new_name="a2",
        )
        api_instance = api.API()
        result = api_instance.rename(args)
        ```
        """
        return self.call("KclvmService.Rename", args)

    def rename_code(self, args: RenameCode_Args) -> RenameCode_Result:
        """Rename all the occurrences of the target symbol and return the modified code if any code has been changed. This API won't rewrite files but return the changed code.

        ## Example

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.RenameCode_Args(
            package_root="/mock/path",
            symbol_path="a",
            source_codes={"/mock/path/main.k": "a = 1"},
            new_name="a2",
        )
        api_instance = api.API()
        result = api_instance.rename_code(args)
        assert result.changed_codes["/mock/path/main.k"] == "a2 = 1"
        ```
        """
        return self.call("KclvmService.RenameCode", args)

    def test(self, args: Test_Args) -> Test_Result:
        """Test KCL packages with test arguments.

        ## Example

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.Test_Args(
            pkg_list=["path/to/testing/pkg/..."],
        )
        api_instance = api.API()
        result = api_instance.test(args)
        ```
        """
        return self.call("KclvmService.Test", args)

    def update_dependencies(
        self, args: UpdateDependencies_Args
    ) -> UpdateDependencies_Result:
        """Download and update dependencies defined in the `kcl.mod` file and return the external package name and location list.

        ## Examples

        The content of `module/kcl.mod` is

        ```yaml
        [package]
        name = "mod_update"
        edition = "0.0.1"
        version = "0.0.1"

        [dependencies]
        helloworld = { oci = "oci://ghcr.io/kcl-lang/helloworld", tag = "0.1.0" }
        flask = { git = "https://github.com/kcl-lang/flask-demo-kcl-manifests", commit = "ade147b" }
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.UpdateDependencies_Args(
            manifest_path="module"
        )
        api_instance = api.API()
        result = api_instance.update_dependencies(args)
        pkg_names = [pkg.pkg_name for pkg in result.external_pkgs]
        assert len(pkg_names) == 2
        assert "helloworld" in pkg_names
        assert "flask" in pkg_names
        ```

        Call `exec_program` with external dependencies

        The content of `module/kcl.mod` is

        ```yaml
        [package]
        name = "mod_update"
        edition = "0.0.1"
        version = "0.0.1"

        [dependencies]
        helloworld = { oci = "oci://ghcr.io/kcl-lang/helloworld", tag = "0.1.0" }
        flask = { git = "https://github.com/kcl-lang/flask-demo-kcl-manifests", commit = "ade147b" }
        ```

        The content of `module/main.k` is

        ```python
        import helloworld
        import flask
        a = helloworld.The_first_kcl_program
        ```

        Python Code

        ```python
        import kcl_lib.api as api
        args = api.UpdateDependencies_Args(
            manifest_path="module"
        )
        api_instance = api.API()
        result = api_instance.update_dependencies(args)
        exec_args = api.ExecProgram_Args(
            k_filename_list=["module/main.k"],
            external_pkgs=result.external_pkgs,
        )
        result = api_instance.exec_program(exec_args)
        assert result.yaml_result == "a: Hello World!"
        ```
        """
        return self.call("KclvmService.UpdateDependencies", args)

    def get_version(self) -> GetVersion_Result:
        """Return the KCL service version information.

        ## Example

        Python Code

        ```python
        import kcl_lib.api as api

        api_instance = api.API()
        result = api_instance.get_version()
        print(result.version_info)
        ```
        """
        return self.call("KclvmService.GetVersion", GetVersion_Args())

    # Helper method to perform the call
    def call(self, name: str, args):
        """Call KCL API with the API name and argument protobuf bytes."""
        # Serialize arguments using pickle or json
        args_serialized = args.SerializeToString()
        # Call the service function and get the result
        result = bytes(
            kcl_lib.call_with_plugin_agent(
                name.encode("utf-8"), args_serialized, self.plugin_agent
            )
        )
        if result.startswith(b"ERROR:"):
            raise Exception(result.decode(encoding="utf-8").removeprefix("ERROR:"))
        msg = self.create_method_resp_message(name)
        msg.ParseFromString(result)
        return msg

    def create_method_req_message(self, method: str) -> _message.Message:
        if method in ["Ping", "KclvmService.Ping"]:
            return Ping_Args()
        elif method in ["ExecProgram", "KclvmService.ExecProgram"]:
            return ExecProgram_Args()
        elif method in ["BuildProgram", "KclvmService.BuildProgram"]:
            return BuildProgram_Args()
        elif method in ["ExecArtifact", "KclvmService.ExecArtifact"]:
            return ExecArtifact_Args()
        elif method in ["ParseFile", "KclvmService.ParseFile"]:
            return ParseFile_Args()
        elif method in ["ParseProgram", "KclvmService.ParseProgram"]:
            return ParseProgram_Args()
        elif method in ["LoadPackage", "KclvmService.LoadPackage"]:
            return LoadPackage_Args()
        elif method in ["ListOptions", "KclvmService.ListOptions"]:
            return ParseProgram_Args()
        elif method in ["ListVariables", "KclvmService.ListVariables"]:
            return ListVariables_Args()
        elif method in ["FormatCode", "KclvmService.FormatCode"]:
            return FormatCode_Args()
        elif method in ["FormatPath", "KclvmService.FormatPath"]:
            return FormatPath_Args()
        elif method in ["LintPath", "KclvmService.LintPath"]:
            return LintPath_Args()
        elif method in ["OverrideFile", "KclvmService.OverrideFile"]:
            return OverrideFile_Args()
        elif method in ["GetSchemaTypeMapping", "KclvmService.GetSchemaTypeMapping"]:
            return GetSchemaTypeMapping_Args()
        elif method in ["ValidateCode", "KclvmService.ValidateCode"]:
            return ValidateCode_Args()
        elif method in ["ListDepFiles", "KclvmService.ListDepFiles"]:
            return ListDepFiles_Args()
        elif method in ["LoadSettingsFiles", "KclvmService.LoadSettingsFiles"]:
            return LoadSettingsFiles_Args()
        elif method in ["Rename", "KclvmService.Rename"]:
            return Rename_Args()
        elif method in ["RenameCode", "KclvmService.RenameCode"]:
            return RenameCode_Args()
        elif method in ["Test", "KclvmService.Test"]:
            return Test_Args()
        elif method in ["UpdateDependencies", "KclvmService.UpdateDependencies"]:
            return UpdateDependencies_Args()
        elif method in ["GetVersion", "KclvmService.GetVersion"]:
            return GetVersion_Args()
        raise Exception(f"unknown method: {method}")

    def create_method_resp_message(self, method: str) -> _message.Message:
        if method in ["Ping", "KclvmService.Ping"]:
            return Ping_Result()
        elif method in ["ExecProgram", "KclvmService.ExecProgram"]:
            return ExecProgram_Result()
        elif method in ["BuildProgram", "KclvmService.BuildProgram"]:
            return BuildProgram_Result()
        elif method in ["ExecArtifact", "KclvmService.ExecArtifact"]:
            return ExecProgram_Result()
        elif method in ["ParseFile", "KclvmService.ParseFile"]:
            return ParseFile_Result()
        elif method in ["ParseProgram", "KclvmService.ParseProgram"]:
            return ParseProgram_Result()
        elif method in ["LoadPackage", "KclvmService.LoadPackage"]:
            return LoadPackage_Result()
        elif method in ["ListOptions", "KclvmService.ListOptions"]:
            return ListOptions_Result()
        elif method in ["ListVariables", "KclvmService.ListVariables"]:
            return ListVariables_Result()
        elif method in ["FormatCode", "KclvmService.FormatCode"]:
            return FormatCode_Result()
        elif method in ["FormatPath", "KclvmService.FormatPath"]:
            return FormatPath_Result()
        elif method in ["LintPath", "KclvmService.LintPath"]:
            return LintPath_Result()
        elif method in ["OverrideFile", "KclvmService.OverrideFile"]:
            return OverrideFile_Result()
        elif method in ["GetSchemaTypeMapping", "KclvmService.GetSchemaTypeMapping"]:
            return GetSchemaTypeMapping_Result()
        elif method in ["ValidateCode", "KclvmService.ValidateCode"]:
            return ValidateCode_Result()
        elif method in ["ListDepFiles", "KclvmService.ListDepFiles"]:
            return ListDepFiles_Result()
        elif method in ["LoadSettingsFiles", "KclvmService.LoadSettingsFiles"]:
            return LoadSettingsFiles_Result()
        elif method in ["Rename", "KclvmService.Rename"]:
            return Rename_Result()
        elif method in ["RenameCode", "KclvmService.RenameCode"]:
            return RenameCode_Result()
        elif method in ["Test", "KclvmService.Test"]:
            return Test_Result()
        elif method in ["UpdateDependencies", "KclvmService.UpdateDependencies"]:
            return UpdateDependencies_Result()
        elif method in ["GetVersion", "KclvmService.GetVersion"]:
            return GetVersion_Result()
        raise Exception(f"unknown method: {method}")
